import React__default,{useState,useMemo,useEffect,useCallback}from'react';import {useFela}from'react-fela';import {message,Card,Switch,Dropdown,Button,DatePicker,Menu,Row,Col,Space}from'antd';import {CaretDownOutlined}from'@ant-design/icons';import moment from'moment';import {BarChart}from'./BarChart.js';import {DoughnutChart}from'./DoughnutChart.js';import {roundDuration,formatDuration}from'../timetracker/services/timeRecordService.js';import {useRound,useTimeformat}from'../timetracker/hooks/index.js';import {ExportReportAsExcel}from'./ExportReportAsExcel.js';import {ExportReportAsCSV}from'./ExportReportAsCSV.js';import {ExportReportAsPDF}from'./ExportReportAsPDF.js';import'tslib';import {calculateCost,calculateProfit}from'../utils/calculate.js';import {ReportTable}from'./ReportTable.js';import {Timesheet}from'./Timesheet/Timesheet.js';import*as _ from'lodash';const { RangePicker } = DatePicker;
var ShowType;
(function (ShowType) {
    ShowType["SHOW_AMOUNT"] = "Show amount";
    ShowType["SHOW_COST"] = "Show cost";
    ShowType["SHOW_PROFIT"] = "Show profit";
    ShowType["HIDE_AMOUNT"] = "Hide amount";
})(ShowType || (ShowType = {}));
var GroupBy;
(function (GroupBy) {
    GroupBy["USER"] = "user";
    GroupBy["DESCRIPTION"] = "description";
    GroupBy["PROJECT"] = "project";
    GroupBy["TAG"] = "tag";
})(GroupBy || (GroupBy = {}));
const Reports = ({ range, projects, records, recordsByUserId, setRange, updateConfiguration, users }) => {
    const [isRounded, setIsRounded] = useState(false);
    const [showType, setShowType] = useState(ShowType.SHOW_AMOUNT);
    const [groupBy, setGroupBy] = useState(GroupBy.PROJECT);
    const [groupByRecords, setGroupByRecords] = useState([]);
    const { roundType, roundValue, rounded, refetchRounded } = useRound();
    const { dateFormat, timeFormat } = useTimeformat();
    const { css } = useFela();
    const debounceFunc = useMemo(() => _.debounce((checked) => {
        updateConfiguration({ updateKey: 'timetracker.report.timeRoundingInReports', value: checked })
            .then(async () => {
            await refetchRounded();
            message.success('Rounded setting updated');
        })
            .catch((e) => {
            console.log(e.message);
        });
    }, 1000), [updateConfiguration]);
    const handleSwitchRoundMode = (checked) => {
        setIsRounded(checked);
        debounceFunc(checked);
    };
    const handleChangeRange = (range, str) => {
        const start = moment(range[0]);
        const end = moment(range[1]);
        setRange({ start, end });
    };
    useEffect(() => {
        setIsRounded(rounded);
    }, [rounded]);
    useEffect(() => {
        let list = [];
        switch (groupBy) {
            case GroupBy.PROJECT:
                list = _.chain(records)
                    .groupBy('projectId')
                    .map((value, key) => {
                    var _a;
                    const name = ((_a = projects === null || projects === void 0 ? void 0 : projects.find(p => (p.id === key))) === null || _a === void 0 ? void 0 : _a.name) || 'Unknown';
                    return ({ id: key, name, records: value });
                })
                    .value();
                break;
            case GroupBy.USER:
                list = _.chain(records)
                    .groupBy('userId')
                    .map((value, key) => {
                    var _a;
                    const name = ((_a = users === null || users === void 0 ? void 0 : users.find((u) => (u.userId === key))) === null || _a === void 0 ? void 0 : _a.name) || '(Without user)';
                    return ({ id: key, name, records: value });
                })
                    .value();
                break;
            case GroupBy.DESCRIPTION:
                list = _.chain(records)
                    .groupBy('description')
                    .map((value, key) => {
                    const name = (key && key !== 'null') ? key : '(Without description)';
                    return ({ id: key, name, records: value });
                })
                    .value();
                break;
            case GroupBy.TAG:
                list = _.chain(records)
                    .groupBy('tags')
                    .map((value, key) => {
                    return ({ id: key, name: key || '(Without tag)', records: value });
                })
                    .value();
                break;
        }
        setGroupByRecords(list);
    }, [groupBy, records, projects, users]);
    const generateLabels = () => {
        const daysCnt = Math.abs(moment(range.end).diff(range.start, 'days'));
        const labels = Array(daysCnt)
            .fill(0)
            .map((itemValue, itemIndex) => {
            return moment(range.start).add(itemIndex, 'day').format('ddd, MMM D');
        });
        return labels;
    };
    const calcDurationReducer = (totalDur, record) => totalDur + Math.abs(Math.floor((moment(record.endTime).valueOf() - moment(record.startTime).valueOf()) / 1000));
    const generateBarData = () => {
        const daysCnt = Math.abs(moment(range.start).diff(range.end, 'days'));
        const dataSet = Array(daysCnt)
            .fill(0)
            .map((itemValue, itemIndex) => {
            const curDayStr = moment(range.start).add(itemIndex, 'day').format(dateFormat);
            const curDayRecords = records.filter((r) => curDayStr === moment(r.startTime).format(dateFormat));
            const totalDuration = curDayRecords.reduce(calcDurationReducer, 0);
            return rounded ? roundDuration(totalDuration, roundValue, roundType) : totalDuration;
        });
        return dataSet;
    };
    const generateProjectLabels = () => {
        const pLabels = groupByRecords.map((item, index) => {
            const pTotalDur = item.records.reduce(calcDurationReducer, 0);
            const duration = rounded ? roundDuration(pTotalDur, roundValue, roundType) : pTotalDur;
            return `${item.name} - ${formatDuration(duration, timeFormat)}`;
        });
        return pLabels;
    };
    const generateProjectDurations = () => {
        const pDurArray = groupByRecords.map((item, index) => {
            const pTotalDur = item.records.reduce(calcDurationReducer, 0);
            return rounded ? roundDuration(pTotalDur, roundValue, roundType) : pTotalDur;
        });
        return pDurArray;
    };
    const getBillableDuration = (records) => {
        const billableRecords = records.filter((record) => record.isBillable);
        const totalDur = billableRecords.reduce(calcDurationReducer, 0);
        return rounded ? roundDuration(totalDur, roundValue, roundType) : totalDur;
    };
    const generateTableColumns = () => {
        return [
            {
                key: 'title',
                dataIndex: 'title',
                title: 'Title',
            },
            {
                key: 'duration',
                dataIndex: 'duration',
                title: 'Duration',
                render: (value) => {
                    return React__default.createElement("span", null,
                        " ",
                        formatDuration(value, timeFormat));
                },
            },
        ];
    };
    const generateDatasource = () => {
        const pDuration = groupByRecords.map((item, index) => {
            const pTotalDur = item.records.reduce(calcDurationReducer, 0);
            return {
                id: index,
                title: item.name,
                duration: rounded ? roundDuration(pTotalDur, roundValue, roundType) : pTotalDur,
            };
        });
        return pDuration;
    };
    const saveAsPdf = () => {
        const input = document.getElementById('projects-report');
        if (input) {
            const html2PDF = require('jspdf-html2canvas');
            html2PDF(input, {
                jsPDF: {
                    format: 'a4',
                },
                margin: {
                    top: 40,
                    right: 30,
                    bottom: 40,
                    left: 30,
                },
                imageType: 'image/jpeg',
                output: 'report_pdf.pdf',
            });
        }
    };
    const exportMenu = () => {
        const menu = (React__default.createElement(Menu, null,
            React__default.createElement(Menu.Item, { onClick: saveAsPdf }, "Save as PDF"),
            React__default.createElement(Menu.Item, null,
                React__default.createElement(ExportReportAsCSV, { groupBy: groupBy, groupByRecords: groupByRecords, rounded: rounded, roundValue: roundValue, roundType: roundType, calcDurationReducer: calcDurationReducer })),
            React__default.createElement(Menu.Item, null,
                React__default.createElement(ExportReportAsExcel, { range: range, groupBy: groupBy, groupByRecords: groupByRecords, rounded: rounded, roundValue: roundValue, roundType: roundType, calcDurationReducer: calcDurationReducer }))));
        return (React__default.createElement(Dropdown, { overlay: menu, placement: "bottomCenter" },
            React__default.createElement(Button, null,
                React__default.createElement("span", null, "Export"),
                React__default.createElement(CaretDownOutlined, null))));
    };
    const handleMenuClick = (e) => {
        setShowType(e.key);
    };
    const showAmount = () => {
        const menu = (React__default.createElement(Menu, { onClick: handleMenuClick },
            React__default.createElement(Menu.Item, { key: ShowType.SHOW_AMOUNT }, ShowType.SHOW_AMOUNT),
            React__default.createElement(Menu.Item, { key: ShowType.SHOW_COST }, ShowType.SHOW_COST),
            React__default.createElement(Menu.Item, { key: ShowType.SHOW_PROFIT }, ShowType.SHOW_PROFIT),
            React__default.createElement(Menu.Item, { key: ShowType.HIDE_AMOUNT }, ShowType.HIDE_AMOUNT)));
        return (React__default.createElement(Dropdown, { overlay: menu, placement: "bottomCenter" },
            React__default.createElement(Button, null,
                React__default.createElement("span", null, showType),
                React__default.createElement(CaretDownOutlined, null))));
    };
    const generateRanges = () => [
        {
            label: 'Today',
            start: moment().startOf('day'),
            end: moment().endOf('day'),
        },
        {
            label: 'Yesterday',
            start: moment().add(-1, 'day').startOf('day'),
            end: moment().add(-1, 'day').endOf('day'),
        },
        {
            label: 'This week',
            start: moment().startOf('week'),
            end: moment().endOf('week'),
        },
        {
            label: 'Last week',
            start: moment().add(-1, 'week').startOf('week'),
            end: moment().add(-1, 'week').endOf('week'),
        },
        {
            label: 'This month',
            start: moment().startOf('month'),
            end: moment().endOf('month'),
        },
        {
            label: 'Last month',
            start: moment().add(-1, 'month').startOf('month'),
            end: moment().add(-1, 'month').endOf('month'),
        },
    ];
    const handleClickRange = (range) => {
        setRange({
            start: moment(range.start),
            end: moment(range.end),
        });
    };
    const getAmount = useCallback(() => {
        var _a;
        let amount = 0;
        let cost = 0;
        let profit = 0;
        (_a = Object.values(recordsByUserId)) === null || _a === void 0 ? void 0 : _a.forEach((record) => {
            const { data, payRate = 0, billRate = 0 } = record;
            if (data) {
                const billable = getBillableDuration(data);
                const hours = (billable / 3600);
                amount += (hours * payRate);
                cost += calculateCost(payRate, hours);
                profit += calculateProfit(payRate, billRate, hours);
            }
        });
        return { amount, cost, profit };
    }, [recordsByUserId]);
    const totalTime = generateProjectDurations().length &&
        generateProjectDurations().reduce((accumulator, currentValue) => accumulator + currentValue);
    const getHeader = (labelStyle, valueStyle) => {
        const { amount, cost, profit } = getAmount();
        switch (showType) {
            case ShowType.SHOW_AMOUNT:
                return (React__default.createElement(React__default.Fragment, null,
                    React__default.createElement("div", null,
                        React__default.createElement("span", { className: css(labelStyle) }, "Billable:"),
                        React__default.createElement("span", { className: css(valueStyle) }, formatDuration(getBillableDuration(records), timeFormat))),
                    React__default.createElement("div", null,
                        React__default.createElement("span", { className: css(labelStyle) }, "Amount:"),
                        React__default.createElement("span", { className: css(valueStyle) },
                            amount.toFixed(2),
                            " USD"))));
            case ShowType.SHOW_COST:
                return (React__default.createElement("div", null,
                    React__default.createElement("span", { className: css(labelStyle) }, "Cost:"),
                    React__default.createElement("span", { className: css(valueStyle) },
                        cost.toFixed(2),
                        " USD")));
            case ShowType.SHOW_PROFIT:
                return (React__default.createElement("div", null,
                    React__default.createElement("span", { className: css(labelStyle) }, "Profit:"),
                    React__default.createElement("span", { className: css(valueStyle) }, profit === null || profit === void 0 ? void 0 :
                        profit.toFixed(2),
                        " USD")));
            default:
                return null;
        }
    };
    const header = (React__default.createElement("div", { className: css(styles.header) },
        React__default.createElement("div", null,
            React__default.createElement("span", { className: css(styles.label) }, "Total:"),
            React__default.createElement("span", { className: css(styles.value) }, formatDuration(totalTime, timeFormat))),
        getHeader(styles.label, styles.value)));
    const panelRender = (originalPanel) => {
        return (React__default.createElement(Row, null,
            React__default.createElement(Col, { xs: 24, md: 5 },
                React__default.createElement(Space, { direction: "vertical", style: { width: '100%' } }, generateRanges().map((range) => {
                    return (React__default.createElement(Button, { onClick: () => handleClickRange(range), key: range.label, style: { width: '100%' } }, range.label));
                }))),
            React__default.createElement(Col, { xs: 24, md: 18, style: { paddingRight: '50px' } }, originalPanel)));
    };
    return (React__default.createElement("div", { className: css(styles.container) },
        React__default.createElement(Card, { title: 'Timetracker Report', bordered: false, extra: header },
            React__default.createElement("div", { className: css(styles.barTitle, styles.fullWidth) },
                React__default.createElement("div", null,
                    React__default.createElement(RangePicker, { onChange: handleChangeRange, panelRender: panelRender, value: [range.start, range.end] })),
                React__default.createElement("div", { className: css(styles.barTitle, styles.right) },
                    React__default.createElement("div", { className: css(styles.roundWrapper) },
                        React__default.createElement("span", { className: css(styles.roundingLabel) }, "Rounding:"),
                        React__default.createElement(Switch, { size: 'small', checked: isRounded, onChange: handleSwitchRoundMode })),
                    React__default.createElement("div", { className: css(styles.amount) }, showAmount()))),
            React__default.createElement("div", { className: css(styles.mT10) }, exportMenu()),
            React__default.createElement("div", { className: css(styles.title) },
                React__default.createElement("span", { className: "duration-start" },
                    " ",
                    moment(range.start).format('MMMM DD')),
                " -",
                React__default.createElement("span", { className: "duration-end" }, moment(range.start).format('MM') === moment(range.end).format('MM')
                    ? moment(range.end).format('DD')
                    : moment(range.end).format('MMMM DD'))),
            React__default.createElement("div", { className: css(styles.barChartWrapper) },
                React__default.createElement(BarChart, { title: "Reports", data: generateBarData(), labels: generateLabels() }))),
        React__default.createElement("div", { className: css(styles.flexM) },
            React__default.createElement(ReportTable, { groupBy: groupBy, setGroupBy: setGroupBy, generateDatasource: generateDatasource, generateTableColumns: generateTableColumns }),
            React__default.createElement(Card, { className: css(styles.chartCard), title: 'Report' },
                React__default.createElement(DoughnutChart, { title: "Reports", data: generateProjectDurations(), labels: generateProjectLabels() }))),
        React__default.createElement(Timesheet, null),
        React__default.createElement(ExportReportAsPDF, { range: range, generateBarData: generateBarData, generateLabels: generateLabels, generateProjectDurations: generateProjectDurations, generateProjectLabels: generateProjectLabels, generateTableColumns: generateTableColumns, generateDatasource: generateDatasource, getHeader: getHeader })));
};
const styles = {
    container: () => ({
        '& .ant-table-cell': {
            padding: '11px',
        },
        '& .ant-card': {
            width: '100%',
            overflowX: 'auto',
        },
    }),
    fullWidth: () => ({
        width: '100%',
    }),
    flex: () => ({
        display: 'flex',
    }),
    flexM: () => ({
        '@media (min-width: 800px)': {
            display: 'flex',
        },
    }),
    roundingLabel: () => ({
        margin: '-1px 6px',
        fontWeight: 400,
    }),
    chartCard: () => ({
        '@media (min-width: 800px)': {
            width: '50%',
        },
        marginTop: '25px',
    }),
    barChartWrapper: () => ({
        margin: '0 auto',
        '@media (min-width: 1200px)': {
            width: '80%',
        },
    }),
    right: () => ({
        '@media (min-width: 683px)': {
            marginLeft: 'auto',
        },
    }),
    title: () => ({
        width: '100%',
        textAlign: 'center',
    }),
    label: () => ({
        color: '#696868',
        marginLeft: '20px',
        marginRight: '5px',
    }),
    value: () => ({
        fontSize: '16px',
        fontWeight: '500',
    }),
    mT10: () => ({
        marginTop: '10px',
    }),
    header: () => ({
        '@media (max-width: 653px)': {
            display: 'inline-grid',
        },
        '@media (min-width: 653px)': {
            display: 'inline-flex',
        },
    }),
    barTitle: () => ({
        '@media (max-width: 683px)': {
            display: 'inline-grid',
        },
        '@media (min-width: 683px)': {
            display: 'inline-flex',
        },
    }),
    roundWrapper: () => ({
        marginTop: 'auto',
        marginBottom: 'auto',
        marginRight: '10px',
        '@media (max-width: 683px)': {
            marginTop: '10px',
        },
    }),
    amount: () => ({
        '@media (max-width: 683px)': {
            marginTop: '10px',
        },
    })
};export{GroupBy,Reports};//# sourceMappingURL=ReportComponent.js.map
