import React__default,{useState,useEffect,useRef}from'react';import {useDispatch,useSelector}from'react-redux';import {FieldTimeOutlined}from'@ant-design/icons';import Draggable from'react-draggable';import {Button,message}from'antd';import {useFela}from'react-fela';import {merge,omit}from'lodash';import moment from'moment';import {styleSheet}from'./style.js';import TimerWidget from'./TimerWidget/index.js';import {currentTimerSelector,setCurrentTimerAction}from'@admin-layout/timetracker-core';import {useGetPlayingTimeRecordQuery,useCreateTimeRecordMutation,useUpdateTimeRecordMutation,SubscribeToTimeTrackerDocument}from'@admin-layout/timetracker-core/lib/generated/react-hooks';import {useCreatePermissions}from'../../hooks/index.js';import {TimerComponent}from'../TimeTracker/TimerComponent.js';import {useInjection}from'@workbench-stack/components';import {generateContributionId,CONTRIBUTION_ACTION_TYPES}from'@adminide-stack/extension-api';const SetCurrentTimer = () => {
    const activityService = useInjection("ActivtyService" /* ActivtyService */);
    const currentTimer = useSelector(currentTimerSelector);
    const userId = useSelector((state) => state.user.auth0UserId);
    const currentTimeRecordRef = useRef(currentTimer);
    const [updateMutation] = useUpdateTimeRecordMutation();
    const dispatch = useDispatch();
    useEffect(() => {
        console.log('---TIMER-----', currentTimeRecordRef);
        //pause or restart timer on timeout and activity trigger
        activityService.onTimeoutKeyup((value) => {
            var _a;
            const { current: currentTimeRecord } = currentTimeRecordRef;
            console.log('---TIMER2-----', currentTimeRecord);
            if (currentTimeRecord.startTime !== null) {
                let pauseStatus = (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime, status }) => ({ pausedTime, restartTime, status }));
                if (value) {
                    //pause timer
                    if (pauseStatus) {
                        pauseStatus.push({ pausedTime: moment(), restartTime: null, status: "DUE_TO_INACTIVITY" /* DUE_TO_INACTIVITY */ });
                    }
                    else {
                        pauseStatus = [{ pausedTime: moment(), restartTime: null, status: "DUE_TO_INACTIVITY" /* DUE_TO_INACTIVITY */ }];
                    }
                }
                else {
                    //restart timer
                    if (pauseStatus && pauseStatus.length) {
                        pauseStatus[pauseStatus.length - 1] = Object.assign(Object.assign({}, pauseStatus[pauseStatus.length - 1]), { restartTime: moment(), status: "DUE_TO_INACTIVITY" /* DUE_TO_INACTIVITY */ });
                    }
                }
                dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, currentTimeRecord), { isTimeout: value, isPause: value, pauseStatus })));
                updateMutation({
                    variables: {
                        recordId: currentTimeRecord.id,
                        request: Object.assign(Object.assign({}, omit(currentTimeRecord, ['id', '__typename'])), { isTimeout: value, isPause: value, pauseStatus }),
                    },
                    update: (cache, { data }) => {
                        if (data) {
                            message.success('Update TimeRecord');
                        }
                    },
                });
            }
        });
        // auto stop timer when maximum track length exceeds
        activityService.onStopTimer((value) => {
            var _a;
            if (value) {
                const { current: currentTimeRecord } = currentTimeRecordRef;
                const endTime = moment();
                const startTime = moment(currentTimeRecord.startTime);
                const newTimeRecord = {
                    userId: userId,
                    startTime: startTime,
                    endTime: endTime,
                    description: currentTimeRecord.description,
                    projectId: currentTimeRecord.projectId,
                    isBillable: currentTimeRecord.isBillable,
                    taskName: currentTimeRecord.taskName,
                    isPause: currentTimeRecord.isPause,
                    isTimeout: currentTimeRecord.isTimeout,
                    pauseStatus: (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime, status }) => ({ pausedTime, restartTime, status })),
                };
                updateMutation({
                    variables: { recordId: currentTimeRecord.id, request: Object.assign({}, newTimeRecord) },
                    update: (cache, { data }) => {
                        if (data) {
                            message.success('Stopped Timer');
                        }
                    },
                });
            }
        });
    }, []);
    useEffect(() => {
        currentTimeRecordRef.current = Object.assign({}, currentTimer);
    }, [currentTimer]);
    return null;
};
const HeaderTimerHandler = (props) => {
    const { css } = useFela();
    const [visiblity, setVisiblity] = useState(false);
    const dispatch = useDispatch();
    const [startTimer, setStartTimer] = useState(false);
    const [currentTimeRecord, setCurrentTimeRecord] = useState({
        startTime: null,
        endTime: null,
        isBillable: false,
        description: '',
        projectId: '',
        taskName: '',
        isPause: false,
        isTimeout: false,
        pauseStatus: null,
    });
    const userId = useSelector((state) => state.user.auth0UserId);
    const orgName = useSelector((state) => state.platform.orgName);
    useSelector(currentTimerSelector);
    const { data: plData, loading: plLoading, subscribeToMore, refetch } = useGetPlayingTimeRecordQuery();
    const [createMutation] = useCreateTimeRecordMutation();
    const [updateMutation] = useUpdateTimeRecordMutation();
    const { self: createPermit } = useCreatePermissions();
    useEffect(() => {
        const id = generateContributionId();
        const contribution = {
            [id]: {
                pageNavBar: [
                    { name: 'dispatch-action', position: 'right', component: () => React__default.createElement(SetCurrentTimer, null) },
                ],
            },
        };
        // register contribution
        dispatch({
            type: CONTRIBUTION_ACTION_TYPES.REGISTER_CONTRIBUTIONS,
            payload: contribution,
        });
        return () => {
            dispatch({
                type: CONTRIBUTION_ACTION_TYPES.DEREGISTER_CONTRIBUTIONS,
                payload: contribution,
            });
        };
    }, []);
    useEffect(() => {
        if (startTimer) {
            resetTimerValues();
        }
        refetch();
    }, [orgName]);
    const hidePopover = () => {
        setVisiblity(false);
    };
    useEffect(() => {
        const unsubscribe = subscribeToMore(getSubscriptionOptions({ userId, orgName }));
        return () => unsubscribe();
    }, [orgName, userId, subscribeToMore]);
    useEffect(() => {
        if (!plLoading) {
            const playingRecord = plData === null || plData === void 0 ? void 0 : plData.getPlayingTimeRecord;
            if (playingRecord && playingRecord.startTime) {
                setCurrentTimeRecord(playingRecord);
                dispatch(setCurrentTimerAction(Object.assign({}, playingRecord)));
                setStartTimer(true);
            }
            else if (!playingRecord || (playingRecord && startTimer)) {
                resetTimerValues();
            }
            else if (playingRecord) {
                setCurrentTimeRecord(playingRecord);
            }
        }
    }, [plData, plLoading]);
    const getSubscriptionOptions = ({ userId, orgName, }) => {
        return {
            document: SubscribeToTimeTrackerDocument,
            variables: { userId, orgName },
            updateQuery: (prev, { subscriptionData }) => {
                var _a, _b;
                const { getPlayingTimeRecord } = prev;
                const subscribedData = subscriptionData.data.SubscribeToTimeTracker;
                if (!subscribedData || (getPlayingTimeRecord &&
                    (subscribedData.timeRecord.id !== getPlayingTimeRecord.id) &&
                    subscribedData.timeRecord.startTime && subscribedData.timeRecord.endTime)) {
                    return prev;
                }
                if ((subscribedData.mutation === "TimeRecordCreated" /* TimeRecordCreated */ ||
                    subscribedData.mutation === "TimeRecordUpdated" /* TimeRecordUpdated */) &&
                    ((_a = subscribedData.timeRecord) === null || _a === void 0 ? void 0 : _a.endTime) === null) {
                    const newData = merge({}, prev, {
                        getPlayingTimeRecord: subscriptionData.data.SubscribeToTimeTracker.timeRecord,
                    });
                    return newData;
                }
                else if (subscribedData.mutation === "TimeRecordUpdated" /* TimeRecordUpdated */ &&
                    ((_b = subscribedData.timeRecord) === null || _b === void 0 ? void 0 : _b.endTime) !== null) {
                    return {
                        getPlayingTimeRecord: {
                            __typename: 'TimeRecord',
                            id: '',
                            description: '',
                            endTime: '',
                            isBillable: false,
                            projectId: '',
                            taskName: '',
                            tags: [],
                            startTime: '',
                            userId: '',
                            isPause: false,
                            isTimeout: false,
                            pauseStatus: null,
                        },
                    };
                }
            },
        };
    };
    // create time record
    const createTimeRecord = (request) => {
        var _a;
        if (createPermit !== "Allow" /* Allow */) {
            message.warning('Permission Not Allowed');
            return;
        }
        request = Object.assign(Object.assign({}, omit(request, ['id', '__typename'])), { pauseStatus: (_a = request.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime })) });
        return createMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.info('TimeRecord created');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    // update time record
    const updateTimeRecord = (recordId, request) => {
        var _a;
        request = Object.assign(Object.assign({}, omit(request, ['id', '__typename'])), { pauseStatus: (_a = request.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({
                pausedTime,
                restartTime,
                status: "BY_USER" /* BY_USER */,
            })) });
        return updateMutation({
            variables: { recordId, request },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.success('TimeRecord Updated');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    const resetTimerValues = () => {
        setStartTimer(false);
        const defaultPlayingRecord = {
            startTime: null,
            description: '',
            endTime: null,
            isBillable: false,
            projectId: '',
            taskName: '',
            isPause: false,
            isTimeout: false,
            pauseStatus: null,
        };
        setCurrentTimeRecord(Object.assign({}, defaultPlayingRecord));
        dispatch(setCurrentTimerAction(Object.assign({}, defaultPlayingRecord)));
    };
    // save current time record to database
    const saveCurrentTimeRecord = () => {
        var _a;
        const endTime = moment();
        const startTime = moment(currentTimeRecord.startTime);
        const newTimeRecord = {
            userId: userId,
            startTime: startTime,
            endTime: endTime,
            description: currentTimeRecord.description,
            projectId: currentTimeRecord.projectId,
            isBillable: currentTimeRecord.isBillable,
            taskName: currentTimeRecord.taskName,
            isPause: currentTimeRecord.isPause,
            isTimeout: currentTimeRecord.isTimeout,
            pauseStatus: (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime })),
        };
        if (currentTimeRecord.id === undefined || currentTimeRecord.id === '') {
            return createTimeRecord(newTimeRecord);
        }
        else {
            return updateTimeRecord(currentTimeRecord.id, newTimeRecord);
        }
    };
    const updatePlayingTimeRecord = (timeRecord) => {
        setCurrentTimeRecord(timeRecord);
        if (currentTimeRecord.id !== undefined && currentTimeRecord.id !== '') {
            updateTimeRecord(timeRecord.id, Object.assign({}, omit(timeRecord, ['__typename', 'id'])));
        }
    };
    const onStartTimer = () => {
        var _a;
        if (!currentTimeRecord.startTime) {
            const newTimeRecord = Object.assign(Object.assign({}, currentTimeRecord), { userId, startTime: moment(), endTime: null });
            return createTimeRecord(newTimeRecord);
        }
        else {
            //restart timer
            let pauseStatus = (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime }));
            if (pauseStatus && pauseStatus.length) {
                pauseStatus[pauseStatus.length - 1] = Object.assign(Object.assign({}, pauseStatus[pauseStatus.length - 1]), { restartTime: moment() });
            }
            dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, currentTimeRecord), { isPause: false, pauseStatus })));
            return updateTimeRecord(currentTimeRecord.id, Object.assign(Object.assign({}, omit(currentTimeRecord, ['__typename', 'id'])), { isPause: false, pauseStatus }));
        }
    };
    const onStopTimer = () => {
        return saveCurrentTimeRecord();
    };
    const onPauseTimer = () => {
        var _a;
        let pauseStatus = (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime }));
        if (pauseStatus) {
            pauseStatus.push({ pausedTime: moment(), restartTime: null });
        }
        else {
            pauseStatus = [{ pausedTime: moment(), restartTime: null }];
        }
        dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, currentTimeRecord), { isPause: true, pauseStatus })));
        return updateTimeRecord(currentTimeRecord.id, Object.assign(Object.assign({}, omit(currentTimeRecord, ['__typename', 'id'])), { isPause: true, pauseStatus }));
    };
    return (React__default.createElement(React__default.Fragment, null,
        React__default.createElement(Draggable, { handle: "._handler", defaultPosition: { x: 0, y: 60 }, position: null, grid: [25, 25], scale: 1, bounds: ".ant-layout:not(.ant-layout-has-sider)", cancel: ".ant-select" },
            React__default.createElement("div", { hidden: !visiblity, className: '_handler', style: { zIndex: 999, position: 'absolute' } },
                React__default.createElement(TimerWidget, { onClose: hidePopover, onStart: onStartTimer, onPause: onPauseTimer, onStop: onStopTimer, trackStarted: startTimer, currentTimeRecord: currentTimeRecord, updatePlayingTimeRecord: updatePlayingTimeRecord, isDraggable: true }))),
        React__default.createElement(Button, { className: css(styleSheet.button), type: startTimer ? 'primary' : 'default', icon: React__default.createElement(FieldTimeOutlined, { className: css(styleSheet.icon) }), onClick: () => setVisiblity(!visiblity) },
            React__default.createElement(TimerComponent, null))));
};export default HeaderTimerHandler;//# sourceMappingURL=index.js.map
