import*as React from'react';import {useState}from'react';import {useFela}from'react-fela';import {message}from'antd';import {EditTwoTone}from'@ant-design/icons';import {useSetting}from'@adminide-stack/platform-client';import {LimitsModal}from'./LimitsModal.js';import {isArray}from'lodash';const Limits = (props) => {
    const [visible, setVisible] = useState(false);
    const { css } = useFela();
    const { data: weeklyLimitConfig, loading: loadWeekly, updateConfiguration } = useSetting({
        configKey: 'timetracker.user.recurringWeeklyLimit',
        overrides: { overrideIdentifier: props.value.record.name },
    });
    const { data: dailyLimitConfig, loading: loadDaily } = useSetting({
        configKey: 'timetracker.user.recurringDailyLimit',
        overrides: { overrideIdentifier: props.value.record.name },
    });
    const { data: daysAllowedConfig, loading } = useSetting({
        configKey: 'timetracker.project.daysAllowedToWork',
        overrides: { overrideIdentifier: props.value.record.name },
    });
    const openLimitsModal = () => {
        setVisible(true);
    };
    const onClose = () => {
        setVisible(false);
    };
    const updateLimitSettings = async (request, daysAllow) => {
        if (weeklyLimit !== parseInt(request.weeklyLimit, 10)) {
            await updateConfiguration({
                updateKey: 'timetracker.user.recurringWeeklyLimit',
                value: parseInt(request.weeklyLimit, 10) || '',
                updateOverrides: { overrideIdentifier: props.value.record.name },
                target: 4 /* ORGANIZATION */,
            });
        }
        if (dailyLimit !== parseInt(request.dailyLimit, 10)) {
            await updateConfiguration({
                updateKey: 'timetracker.user.recurringDailyLimit',
                value: parseInt(request.dailyLimit, 10) || '',
                updateOverrides: { overrideIdentifier: props.value.record.name },
                target: 4 /* ORGANIZATION */,
            });
        }
        if (daysAllowed !== daysAllow.toString()) {
            await updateConfiguration({
                updateKey: 'timetracker.project.daysAllowedToWork',
                value: daysAllow.toString(),
                updateOverrides: { overrideIdentifier: props.value.record.name },
                target: 4 /* ORGANIZATION */,
            });
        }
    };
    const onSubmit = (request, daysAllow, resetFields) => {
        updateLimitSettings(request, daysAllow).then(() => {
            onClose();
            resetFields();
            message.success('Limit details updated successfully');
        });
    };
    if (loading || loadWeekly || loadDaily) {
        return null;
    }
    const weeklyLimit = weeklyLimitConfig === null || weeklyLimitConfig === void 0 ? void 0 : weeklyLimitConfig.resolveConfiguration;
    const dailyLimit = dailyLimitConfig === null || dailyLimitConfig === void 0 ? void 0 : dailyLimitConfig.resolveConfiguration;
    const daysAllowed = daysAllowedConfig === null || daysAllowedConfig === void 0 ? void 0 : daysAllowedConfig.resolveConfiguration;
    return (React.createElement("div", null,
        React.createElement(LimitsModal, { visible: visible, onClose: onClose, onSubmit: onSubmit, weeklyLimit: weeklyLimit, dailyLimit: dailyLimit, daysAllowed: isArray(daysAllowed) ? daysAllowed : daysAllowed.split(',') }),
        React.createElement("div", { className: css(styles.left) },
            weeklyLimit ? React.createElement("div", null, `${weeklyLimit} / week`) : React.createElement("div", { className: css(styles.gray) }, "No weekly limit"),
            dailyLimit ? React.createElement("div", null, `${dailyLimit} / day`) : React.createElement("div", { className: css(styles.gray) }, "No daily limit")),
        React.createElement(EditTwoTone, { className: css(styles.right, styles.m5), onClick: openLimitsModal })));
};
const styles = {
    left: () => ({
        float: 'left',
    }),
    right: () => ({
        float: 'right',
    }),
    gray: () => ({
        color: '#6a6b6c'
    }),
    m5: () => ({
        margin: '5px',
    }),
};export{Limits};//# sourceMappingURL=Limits.js.map
