import React__default,{useState,useEffect,useMemo,useCallback}from'react';import {useFela}from'react-fela';import {PlusCircleOutlined,BarsOutlined,TagOutlined,CaretRightFilled,PauseOutlined,StopFilled,CloseOutlined,ClockCircleOutlined}from'@ant-design/icons';import {Menu,Select,Button,Row,Col,Input,Dropdown,DatePicker,TimePicker,Typography}from'antd';import moment from'moment';import BillableCheck from'../BillableCheck/index.js';import classNames from'classnames';import {TRACKER_MODE}from'../../constants/enums.js';import DurationInput from'../DurationInput/index.js';import {useSelector}from'react-redux';import {useTimeformat}from'../../hooks/index.js';import {TimerComponent}from'./TimerComponent.js';import {ProjectDropdown}from'../ProjectDropdown/ProjectDropdown.js';import*as _ from'lodash';const { RangePicker } = TimePicker;
const { Title } = Typography;
const TimeTracker = (props) => {
    var _a, _b;
    const { isRecording, currentTimeRecord, projects, tasks, mode, disable, setMode, handleStart, handleStop, handlePause, removePlayingTimeRecord, createTimeRecord, updatePlayingTimeRecord, setCurrentTimeRecord, createTask, } = props;
    const { css } = useFela(props);
    const { timeFormat, dateFormat } = useTimeformat();
    const userId = useSelector((state) => state.user.auth0UserId);
    const [manualStart, setManualStart] = useState(moment());
    const [manualEnd, setManualEnd] = useState(moment());
    const [description, setDescription] = useState((_a = currentTimeRecord.description) !== null && _a !== void 0 ? _a : '');
    const [startLoading, setStartLoading] = useState(false);
    const [pauseLoading, setPauseLoading] = useState(false);
    const [stopLoading, setStopLoading] = useState(false);
    useEffect(() => {
        setDescription(currentTimeRecord.description);
    }, [currentTimeRecord.description]);
    const debouncedFunc = useMemo(() => _.debounce((value) => {
        updatePlayingTimeRecord(Object.assign(Object.assign({}, currentTimeRecord), { description: value }));
    }, 800), [currentTimeRecord]);
    const handleChangeDescription = useCallback((e) => {
        e.persist();
        setDescription(e.target.value);
        debouncedFunc(e.target.value);
    }, [debouncedFunc]);
    const handleSelectProjectTask = (projectId, taskName) => {
        updatePlayingTimeRecord(Object.assign(Object.assign({}, currentTimeRecord), { projectId, taskName }));
    };
    const handleChangeBillable = () => {
        updatePlayingTimeRecord(Object.assign(Object.assign({}, currentTimeRecord), { isBillable: !currentTimeRecord.isBillable }));
    };
    const handleTagsChange = (value) => {
        updatePlayingTimeRecord(Object.assign(Object.assign({}, currentTimeRecord), { tags: value }));
    };
    const handleDiscard = (event) => {
        removePlayingTimeRecord();
    };
    const handleChangeMode = (m) => {
        setMode(m);
    };
    const handleChangeRange = (range, str) => {
        const start = range[0];
        const end = range[1];
        setManualStart(start);
        setManualEnd(end);
    };
    const handleChangeDurDate = (date) => {
        console.log('handleChangeDurDate.date =>', date);
    };
    const handleChangeDur = (duration) => {
        setManualEnd(moment(manualStart).add(duration, 'seconds'));
    };
    const handleAddManual = () => {
        const newRecordReq = {
            userId: userId,
            startTime: manualStart,
            endTime: manualEnd,
            isBillable: currentTimeRecord.isBillable,
            taskName: currentTimeRecord.taskName,
            description: currentTimeRecord.description,
            projectId: currentTimeRecord.projectId,
        };
        createTimeRecord(newRecordReq);
    };
    const startTimer = () => {
        setStartLoading(true);
        handleStart(description).then((res) => {
            setStartLoading(false);
        });
    };
    const onClickPause = () => {
        setPauseLoading(true);
        if (currentTimeRecord.isPause) {
            handleStart(description).then((res) => {
                setPauseLoading(false);
            });
        }
        else {
            handlePause().then(() => {
                setPauseLoading(false);
            });
        }
    };
    const onClickStop = () => {
        setStopLoading(true);
        handleStop().then(() => {
            setStopLoading(false);
        });
    };
    // Dropdown for tags
    const tagsOverlay = (React__default.createElement(Menu, null,
        React__default.createElement(Menu.Item, { key: 1 },
            React__default.createElement(Select, { mode: "tags", style: { width: '100%' }, onChange: handleTagsChange, tokenSeparators: [','] }, projects.map((p) => (React__default.createElement(Select.Option, { key: p.id, value: p.id }, p.name)))))));
    const discardConfirmOverlay = (React__default.createElement("div", null,
        "Are you sure to Discard?",
        React__default.createElement(Button, { type: "primary", onClick: handleDiscard }, "Discard")));
    return (React__default.createElement("div", { className: css(styles.timeTracker) },
        React__default.createElement(Row, null,
            React__default.createElement(Col, { span: 24, xxl: 12, className: "input" },
                React__default.createElement(Row, { style: { width: '100%' } },
                    React__default.createElement(Col, { span: 18, className: "flex-center" },
                        React__default.createElement(Input, { placeholder: "What are you working on?", size: "large", value: description, onChange: handleChangeDescription })),
                    React__default.createElement(Col, { span: 6, className: "flex-center project-selection" },
                        React__default.createElement(ProjectDropdown, { projects: projects, tasks: tasks, handleSelectProjectTask: handleSelectProjectTask, createTask: createTask, selectedProjectId: currentTimeRecord.projectId, selectedTask: currentTimeRecord.taskName },
                            React__default.createElement(Button, { icon: currentTimeRecord.projectId === '' ? React__default.createElement(PlusCircleOutlined, null) : React__default.createElement(BarsOutlined, null), style: currentTimeRecord.projectId === '' ? {} : { color: 'green' } }, currentTimeRecord.projectId === ''
                                ? 'Projects'
                                : (_b = projects.find((p) => p.id === currentTimeRecord.projectId)) === null || _b === void 0 ? void 0 : _b.name))))),
            React__default.createElement(Col, { span: 24, xxl: 12, className: "control" },
                React__default.createElement(Row, { style: { width: '100%' } },
                    React__default.createElement(Col, { span: 2, sm: 4, xxl: 2, className: "flex-center" },
                        React__default.createElement(Dropdown, { overlay: tagsOverlay, trigger: ['click'] },
                            React__default.createElement(Button, { icon: React__default.createElement(TagOutlined, null), size: "large" }))),
                    React__default.createElement(Col, { span: 2, sm: 4, xxl: 2, className: classNames({ /*css(styles.billing)*/}, 'flex-center') },
                        React__default.createElement(BillableCheck, { checked: currentTimeRecord.isBillable, onChange: handleChangeBillable })),
                    mode === TRACKER_MODE.TRACK ? (React__default.createElement(React__default.Fragment, null,
                        React__default.createElement(Col, { span: 14, sm: 10, xxl: 14, className: "flex-center" },
                            React__default.createElement(Title, { level: 5, style: { marginBottom: '0px' } },
                                React__default.createElement(TimerComponent, null))),
                        React__default.createElement(Col, { span: 5, sm: 5 },
                            React__default.createElement("div", { className: classNames('start', { hidden: isRecording }, { 'flex-end': !isRecording }) },
                                React__default.createElement(Button, { type: "primary", onClick: startTimer, disabled: disable, loading: startLoading }, "START")),
                            React__default.createElement("div", { className: classNames('start', { hidden: !isRecording }, { 'flex-end': isRecording }) },
                                React__default.createElement("div", { style: { display: 'flex', width: '100%' } },
                                    React__default.createElement("div", { style: { width: '50%' } },
                                        React__default.createElement(Button, { type: "primary", onClick: onClickPause, disabled: disable, loading: pauseLoading },
                                            React__default.createElement("span", { className: css(styles.web) }, currentTimeRecord.isPause ? 'START' : 'PAUSE'),
                                            React__default.createElement("span", { className: css(styles.mobile) }, currentTimeRecord.isPause ? React__default.createElement(CaretRightFilled, null) : React__default.createElement(PauseOutlined, null)))),
                                    React__default.createElement("div", { style: { width: '50%' } },
                                        React__default.createElement(Button, { type: "primary", danger: true, onClick: onClickStop, disabled: disable, loading: stopLoading },
                                            React__default.createElement("span", { className: css(styles.web) }, "STOP"),
                                            React__default.createElement("span", { className: css(styles.mobile) },
                                                React__default.createElement(StopFilled, null))))))),
                        React__default.createElement(Col, { span: 1, className: classNames({ hidden: !isRecording }) },
                            React__default.createElement(Dropdown, { overlay: discardConfirmOverlay, trigger: ['click'] },
                                React__default.createElement(Button, { icon: React__default.createElement(CloseOutlined, null) }))))) : (React__default.createElement(React__default.Fragment, null,
                        React__default.createElement(Col, { span: 16, sm: 6, xxl: 10, className: "flex-row flex-center" },
                            React__default.createElement(RangePicker, { format: timeFormat, defaultValue: [moment(), moment()], bordered: false, onChange: handleChangeRange, value: [manualStart, manualEnd] }),
                            React__default.createElement(DatePicker, { defaultValue: moment(), bordered: false, onChange: handleChangeDurDate, format: dateFormat })),
                        React__default.createElement(Col, { span: 0, sm: 6, className: "duration" },
                            React__default.createElement("div", { className: "flex-center" },
                                React__default.createElement(DurationInput, { duration: Math.floor((moment(manualEnd).valueOf() - moment(manualStart).valueOf()) / 1000), onChange: handleChangeDur }))),
                        React__default.createElement(Col, { span: 3, className: "flex-center" },
                            React__default.createElement(Button, { type: "primary", size: "small", onClick: handleAddManual, disabled: disable }, "ADD")))),
                    React__default.createElement(Col, { span: 1 },
                        React__default.createElement("div", { className: classNames('mode', { hidden: isRecording }, 'flex-col') },
                            React__default.createElement(ClockCircleOutlined, { className: classNames({ selected: mode === TRACKER_MODE.TRACK }), onClick: () => handleChangeMode(TRACKER_MODE.TRACK) }),
                            React__default.createElement(BarsOutlined, { className: classNames({ selected: mode === TRACKER_MODE.MANUAL }), onClick: () => handleChangeMode(TRACKER_MODE.MANUAL) }))))))));
};
const styles = {
    timeTracker: ({ theme }) => ({
        padding: '5px 10px',
        border: '1px solid #eee',
        backgroundColor: '#fff',
        borderRadius: '5px',
        marginBottom: '5px',
        '& .input': {
            display: 'flex',
            flexDirection: 'row',
            justifyContent: 'space-between',
            alignItems: 'center',
            '& .project-selection': {
                overflow: 'hidden',
            },
        },
        '& .control': {
            display: 'flex',
            flexDirection: 'row',
            justifyContent: 'flex-end',
            alignItems: 'center',
            '@media (max-width: 1200px)': {
                justifyContent: 'space-between',
                marginTop: '10px',
                borderTop: '1px solid #eee',
            },
            '& > div': {
                padding: '5px 0px',
            },
            '& .divider': {
                borderLeft: '1px dashed #eee',
                height: '75%',
            },
            '& .start button': {
                width: '100%',
                '@media (max-width: 575px)': {
                    width: '100%',
                },
            },
            '& .hidden': {
                display: 'none',
            },
            '& .project-list': {
                fontStyle: 'bold',
                color: 'red',
            },
            '& .duration': {
                display: 'flex',
                justifyContent: 'center',
                alignItems: 'center',
                '@media (max-width: 576px)': {
                    display: 'none',
                },
            },
        },
        '& .flex-col': {
            display: 'flex',
            flexDirection: 'column',
            justifyContent: 'center',
            alignItems: 'center',
        },
        '& .flex-row': {
            display: 'flex',
            flexDirection: 'row',
            justifyContent: 'center',
            alignItems: 'center',
        },
        '& .flex-center': {
            display: 'flex',
            justifyContent: 'center',
            alignItems: 'center',
        },
        '& .flex-end': {
            display: 'flex',
            justifyContent: 'end',
            alignItems: 'center',
        },
        '& .mode': {
            width: '30px',
        },
        '& .mode .selected': {
            color: '#1890ff',
            border: '1px solid #1890ff',
        },
    }),
    mobile: () => ({
        width: '100%',
        '@media (min-width: 670px)': {
            display: 'none !important',
        },
    }),
    web: () => ({
        width: '100%',
        '@media (max-width: 671px)': {
            display: 'none !important',
        },
    })
};export{TimeTracker};//# sourceMappingURL=index.js.map
