import React__default,{useState,useEffect}from'react';import {Spin,message}from'antd';import TimerWidget from'../HeaderTimerHandler/TimerWidget/index.js';import {useDispatch,useSelector}from'react-redux';import {currentTimerSelector,SubscribeToTimeTrackerDocument,setCurrentTimerAction,calculateTimeDurationSec}from'@admin-layout/timetracker-core';import WeeklyRecord from'./WeeklyRecord/index.js';import {useGetTimeRecordsQuery,useCreateTimeRecordMutation,useUpdateTimeRecordMutation}from'@admin-layout/timetracker-core/lib/generated/react-hooks';import {useCreatePermissions}from'../../hooks/index.js';import moment from'moment';import {omit}from'lodash';const getDate = (startDate, endDate) => {
    if (moment(startDate).format("ddd, Do MMM'YY") === moment(endDate).format("ddd, Do MMM'YY")) {
        return moment(startDate).format("ddd, Do MMM'YY");
    }
    return `${moment(startDate).format("ddd, Do MMM")} - ${moment(endDate).format("ddd, Do MMM'YY")}`;
};
const groupTimeRecords = (records) => {
    if (!records)
        return [];
    let timeRecords = records.slice(0);
    timeRecords.sort((a, b) => {
        if (moment(a.startTime) < moment(b.startTime))
            return 1;
        else if (moment(a.startTime) > moment(b.startTime))
            return -1;
        else
            return 0;
    });
    let groupedDates = {};
    for (let i = 0; i < timeRecords.length; i++) {
        if (timeRecords[i].startTime && timeRecords[i].endTime) {
            const date = moment(timeRecords[i].endTime);
            const dateStr = moment(date).startOf('week').format();
            if (!groupedDates.hasOwnProperty(dateStr)) {
                groupedDates[dateStr] = [];
            }
            groupedDates[dateStr].push(timeRecords[i]);
        }
    }
    let groupedDateArray = [];
    for (let key of Object.keys(groupedDates)) {
        let timeArray = groupedDates[key];
        const startDate = moment(key).startOf('week').format('ddd, MMM DD');
        const endDate = moment(key).endOf('week').format('ddd, MMM DD');
        let totalDurations = 0;
        let groupedDays = {};
        timeArray.forEach((data) => {
            totalDurations += calculateTimeDurationSec(data.startTime, data.endTime, data.pauseStatus);
            const dateString = getDate(data.startTime, data.endTime);
            if (!groupedDays.hasOwnProperty(dateString)) {
                groupedDays[dateString] = [];
            }
            groupedDays[dateString].push(data);
        });
        let groupedDaysArray = [];
        for (let dayKey of Object.keys(groupedDays)) {
            let duration = 0;
            groupedDays[dayKey].forEach((day) => {
                duration += calculateTimeDurationSec(day.startTime, day.endTime, day.pauseStatus);
            });
            groupedDaysArray.push({
                date: dayKey,
                duration,
                days: groupedDays[dayKey],
            });
        }
        groupedDateArray.push({
            dates: `${startDate} - ${endDate}`,
            duration: totalDurations,
            groupedDays: groupedDaysArray,
        });
    }
    return groupedDateArray;
};
const TrayTimerHandler = (props) => {
    const dispatch = useDispatch();
    const currentTimer = useSelector(currentTimerSelector);
    const userId = useSelector((state) => state.user.auth0UserId);
    const orgName = useSelector((state) => state.platform.orgName);
    const [timeRecords, setTimeRecords] = useState([]);
    const [loading, setLoading] = useState(true);
    const [startTimer, setStartTimer] = useState(false);
    const [currentTimeRecord, setCurrentTimeRecord] = useState({
        startTime: null,
        endTime: null,
        isBillable: false,
        description: '',
        projectId: '',
        taskName: '',
        isPause: false,
        isTimeout: false,
        pauseStatus: null,
    });
    const { data, loading: loadingData, refetch, subscribeToMore } = useGetTimeRecordsQuery({ variables: { userId } });
    const [createMutation] = useCreateTimeRecordMutation();
    const [updateMutation] = useUpdateTimeRecordMutation();
    const { self: createPermit } = useCreatePermissions();
    useEffect(() => {
        setTimeout(() => {
            setLoading(false);
        }, 10);
    }, []);
    useEffect(() => {
        refetch();
    }, [orgName]);
    useEffect(() => {
        if (data) {
            setTimeRecords(groupTimeRecords(data.getTimeRecords));
        }
    }, [data]);
    useEffect(() => {
        if (currentTimer && (currentTimer.startTime === null || currentTimer.endTime !== null)) {
            if (startTimer) {
                resetTimerValues();
            }
        }
        else {
            // const passDur = moment().valueOf() - moment(playingRecord.startTime).valueOf();
            setCurrentTimeRecord(currentTimer);
            setStartTimer(true);
        }
    }, [currentTimer]);
    const getSubscriptionDurations = ({ userId, orgName, }) => {
        return {
            document: SubscribeToTimeTrackerDocument,
            variables: { userId, orgName },
            updateQuery: (prev, { subscriptionData }) => {
                var _a;
                const subscribedData = subscriptionData.data.SubscribeToTimeTracker;
                if (!subscribedData) {
                    return prev;
                }
                if (subscribedData.mutation === "TimeRecordUpdated" /* TimeRecordUpdated */ &&
                    ((_a = subscribedData.timeRecord) === null || _a === void 0 ? void 0 : _a.endTime) !== null) {
                    const index = prev.getTimeRecords.findIndex((record) => (record.startTime === subscribedData.timeRecord.startTime));
                    let newData = prev.getTimeRecords.slice(0);
                    if (index === -1) {
                        newData.push(Object.assign({}, subscribedData.timeRecord));
                    }
                    else {
                        newData.splice(index, 1, Object.assign({}, subscribedData.timeRecord));
                    }
                    return { getTimeRecords: newData };
                }
                else if (subscribedData.mutation === "TimeRecordDeleted" /* TimeRecordDeleted */) {
                    const index = prev.getTimeRecords.findIndex((record) => (record.id === subscribedData.timeRecord.id));
                    let newData = prev.getTimeRecords.slice(0);
                    if (index !== -1) {
                        newData.splice(index, 1);
                    }
                    return { getTimeRecords: newData };
                }
            }
        };
    };
    useEffect(() => {
        const unsubscribe = subscribeToMore(getSubscriptionDurations({ userId, orgName }));
        return () => unsubscribe();
    }, [orgName, userId, subscribeToMore]);
    const createTimeRecord = (request) => {
        var _a;
        if (createPermit !== "Allow" /* Allow */) {
            message.warning('Permission Not Allowed');
            return;
        }
        request = Object.assign(Object.assign({}, omit(request, ['id', '__typename'])), { pauseStatus: (_a = request.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime })) });
        return createMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.info('TimeRecord created');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    const updateTimeRecord = (recordId, request) => {
        var _a;
        request = Object.assign(Object.assign({}, omit(request, ['id', '__typename'])), { pauseStatus: (_a = request.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({
                pausedTime, restartTime, status: "BY_USER" /* BY_USER */,
            })) });
        return updateMutation({
            variables: { recordId, request },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.success('TimeRecord Updated');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    const resetTimerValues = () => {
        setStartTimer(false);
        const defaultPlayingRecord = {
            startTime: null,
            description: '',
            endTime: null,
            isBillable: false,
            projectId: '',
            taskName: '',
            isPause: false,
            isTimeout: false,
            pauseStatus: null,
        };
        setCurrentTimeRecord(Object.assign({}, defaultPlayingRecord));
        dispatch(setCurrentTimerAction(Object.assign({}, defaultPlayingRecord)));
    };
    const saveCurrentTimeRecord = () => {
        var _a;
        const endTime = moment();
        const startTime = moment(currentTimeRecord.startTime);
        const newTimeRecord = {
            userId: userId,
            startTime: startTime,
            endTime: endTime,
            description: currentTimeRecord.description,
            projectId: currentTimeRecord.projectId,
            isBillable: currentTimeRecord.isBillable,
            taskName: currentTimeRecord.taskName,
            isPause: currentTimeRecord.isPause,
            isTimeout: currentTimeRecord.isTimeout,
            pauseStatus: (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime })),
        };
        if (currentTimeRecord.id === undefined || currentTimeRecord.id === '') {
            return createTimeRecord(newTimeRecord);
        }
        else {
            return updateTimeRecord(currentTimeRecord.id, newTimeRecord);
        }
    };
    const updatePlayingTimeRecord = (timeRecord) => {
        setCurrentTimeRecord(timeRecord);
        if (currentTimeRecord.id !== undefined && currentTimeRecord.id !== '') {
            return updateTimeRecord(timeRecord.id, Object.assign({}, omit(timeRecord, ['__typename', 'id'])));
        }
    };
    const onStartTimer = () => {
        var _a;
        if (currentTimeRecord.startTime === null) {
            const newTimeRecord = Object.assign(Object.assign({}, currentTimeRecord), { userId, startTime: moment(), endTime: null });
            return createTimeRecord(newTimeRecord);
        }
        else {
            //restart timer
            let pauseStatus = (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime }));
            if (pauseStatus && pauseStatus.length) {
                pauseStatus[pauseStatus.length - 1] = Object.assign(Object.assign({}, pauseStatus[pauseStatus.length - 1]), { restartTime: moment() });
            }
            dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, currentTimeRecord), { isPause: false, pauseStatus })));
            return updateTimeRecord(currentTimeRecord.id, Object.assign(Object.assign({}, omit(currentTimeRecord, ['__typename', 'id'])), { isPause: false, pauseStatus }));
        }
    };
    const onStopTimer = () => {
        return saveCurrentTimeRecord();
    };
    const onPauseTimer = () => {
        var _a;
        let pauseStatus = (_a = currentTimeRecord.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime }));
        if (pauseStatus) {
            pauseStatus.push({ pausedTime: moment(), restartTime: null });
        }
        else {
            pauseStatus = [{ pausedTime: moment(), restartTime: null }];
        }
        dispatch(setCurrentTimerAction(Object.assign(Object.assign({}, currentTimeRecord), { isPause: true, pauseStatus })));
        return updateTimeRecord(currentTimeRecord.id, Object.assign(Object.assign({}, omit(currentTimeRecord, ['__typename', 'id'])), { isPause: true, pauseStatus }));
    };
    let tRecords = [];
    if (!loading) {
        tRecords = timeRecords;
    }
    return (React__default.createElement("div", null,
        React__default.createElement(TimerWidget, { onClose: () => { }, onStart: onStartTimer, onPause: onPauseTimer, onStop: onStopTimer, trackStarted: startTimer, currentTimeRecord: currentTimeRecord, updatePlayingTimeRecord: updatePlayingTimeRecord }),
        React__default.createElement(Spin, { spinning: loading || loadingData },
            React__default.createElement("div", { className: "time-logs" }, tRecords.map((log, index) => React__default.createElement(WeeklyRecord, { key: index, log: log }))))));
};export default TrayTimerHandler;//# sourceMappingURL=index.js.map
