import React__default from'react';import {Table,Dropdown,Button,Menu}from'antd';import {MoreOutlined}from'@ant-design/icons';import moment from'moment';import {useTimeformat,useManagePermissions,useViewPermissions}from'../../hooks/index.js';import {useHistory}from'react-router';import {generatePath}from'react-router-dom';import {useGetOrgContextQuery}from'@adminide-stack/platform-client';import {ROUTES}from'../../constants/routes.js';import'../../constants/enums.js';import*as qs from'query-string';import*as _ from'lodash';import {useSelector}from'react-redux';import {formatDuration}from'../../services/timeRecordService.js';var VIEW_MODE;
(function (VIEW_MODE) {
    VIEW_MODE[VIEW_MODE["OPEN"] = 0] = "OPEN";
    VIEW_MODE[VIEW_MODE["SUBMITTED"] = 1] = "SUBMITTED";
    VIEW_MODE[VIEW_MODE["APPROVED"] = 2] = "APPROVED";
    VIEW_MODE[VIEW_MODE["DENYED"] = 3] = "DENYED";
    VIEW_MODE[VIEW_MODE["ALL"] = 4] = "ALL";
})(VIEW_MODE || (VIEW_MODE = {}));
const TimeReport = ({ timesheets, viewMode, members, updateTimesheet }) => {
    const history = useHistory();
    const { dateFormat } = useTimeformat();
    const { data: contextData } = useGetOrgContextQuery();
    const managePermit = useManagePermissions();
    const viewPermit = useViewPermissions();
    const userId = useSelector((state) => state.user.auth0UserId);
    const handleView = (id, record) => {
        history.push({
            pathname: generatePath(ROUTES.Timesheet, { orgName: contextData.getOrgContext.orgName }),
            search: qs.stringify({
                view: 'tabular',
                weekStart: moment(record.startDate).format('YYYY-MM-DD'),
                username: record.userId,
                strict: 'true',
            }),
        });
    };
    const handleSubmit = (id, record) => {
        const request = Object.assign(Object.assign({}, _.omit(record, ['__typename', 'id', 'orgId', 'totalDuration'])), { submittedOn: moment(), state: "SUBMITTED" /* SUBMITTED */, updatedOn: moment() });
        updateTimesheet(id, request);
    };
    const handleUnSubmit = (id, record) => {
        const request = Object.assign(Object.assign({}, _.omit(record, ['__typename', 'id', 'orgId', 'totalDuration'])), { approvedOn: null, submittedOn: null, state: "OPEN" /* OPEN */, updatedOn: moment() });
        updateTimesheet(id, request);
    };
    const handleApprove = (id, record) => {
        const request = Object.assign(Object.assign({}, _.omit(record, ['__typename', 'id', 'orgId', 'totalDuration'])), { approvedOn: moment(), approvedBy: userId, state: "APPROVED" /* APPROVED */, updatedOn: moment() });
        updateTimesheet(id, request);
    };
    // for testing purpose
    const handleUnApprove = (id, record) => {
        const request = Object.assign(Object.assign({}, _.omit(record, ['__typename', 'id', 'orgId', 'totalDuration'])), { approvedOn: moment(), approvedBy: userId, state: "SUBMITTED" /* SUBMITTED */, updatedOn: moment() });
        updateTimesheet(id, request);
    };
    const handleDeny = (id, record) => {
        const request = Object.assign(Object.assign({}, _.omit(record, ['__typename', 'id', 'orgId', 'totalDuration'])), { state: "DENYED" /* DENYED */, updatedOn: moment() });
        updateTimesheet(id, request);
    };
    const columns = [
        {
            title: 'User',
            dataIndex: 'userId',
            key: 'userId',
            render: (value) => {
                const member = members.find((m) => m.userId === value);
                return React__default.createElement(React__default.Fragment, null,
                    " ",
                    member !== undefined ? member.name : '',
                    " ");
            },
        },
        {
            title: 'Start Date',
            dataIndex: 'startDate',
            key: 'startDate',
            render: (value) => React__default.createElement(React__default.Fragment, null,
                " ",
                moment(value).format(dateFormat || 'YYYY-MM-DD'),
                " "),
        },
        {
            title: 'End Date',
            dataIndex: 'endDate',
            key: 'endDate',
            render: (value) => (React__default.createElement(React__default.Fragment, null, moment(value)
                .add(-1, 'day')
                .format(dateFormat || 'YYYY-MM-DD'))),
        },
        {
            title: 'Duration',
            dataIndex: 'totalDuration',
            key: 'totalDuration',
            render: (value) => React__default.createElement(React__default.Fragment, null,
                " ",
                formatDuration(value),
                " "),
        },
        {
            title: 'Submitted On',
            dataIndex: 'submittedOn',
            key: 'submittedOn',
            render: (value) => React__default.createElement(React__default.Fragment, null,
                " ",
                value ? moment(value).format(dateFormat || 'YYYY-MM-DD') : '-',
                " "),
        },
        {
            title: 'state',
            dataIndex: 'state',
            key: 'state',
        },
        {
            action: 'action',
            key: 'action',
            render: (text, record) => {
                const disableView = (record.userId === userId && viewPermit.self !== "Allow" /* Allow */) ||
                    (record.userId !== userId && viewPermit.others !== "Allow" /* Allow */);
                const disableManage = (record.userId === userId && managePermit.self !== "Allow" /* Allow */) ||
                    (record.userId !== userId && managePermit.others !== "Allow" /* Allow */);
                const actionMenu = () => {
                    return (React__default.createElement(Menu, null,
                        React__default.createElement(Menu.Item, { key: "view", onClick: () => handleView(record.id, record), disabled: disableView }, "View"),
                        viewMode === VIEW_MODE.OPEN && (React__default.createElement(Menu.Item, { key: "submit", onClick: () => handleSubmit(record.id, record) }, "Submit")),
                        viewMode === VIEW_MODE.SUBMITTED && (React__default.createElement(Menu.Item, { key: "unsubmit", onClick: () => handleUnSubmit(record.id, record), disabled: disableManage }, "Unsubmit")),
                        viewMode === VIEW_MODE.SUBMITTED && (React__default.createElement(Menu.Item, { key: "approve", onClick: () => handleApprove(record.id, record), disabled: disableManage }, "Approve")),
                        viewMode === VIEW_MODE.SUBMITTED && (React__default.createElement(Menu.Item, { key: "deny", onClick: () => handleDeny(record.id, record), disabled: disableManage }, "Deny")),
                        viewMode === VIEW_MODE.APPROVED && (React__default.createElement(Menu.Item, { key: "submit", onClick: () => handleUnApprove(record.id, record), disabled: disableManage }, "Unapprove"))));
                };
                return (React__default.createElement(Dropdown, { overlay: actionMenu, trigger: ['click'] },
                    React__default.createElement(Button, null,
                        React__default.createElement(MoreOutlined, null))));
            },
        },
    ];
    return React__default.createElement(Table, { columns: columns, dataSource: timesheets });
};export default TimeReport;//# sourceMappingURL=TimeReportTable.js.map
