import*as React from'react';import {useState,useEffect,useMemo}from'react';import {Spin,Row,Col,Card}from'antd';import Spacer from'../../components/Spacer/index.js';import {CustomScrollbar}from'../../components/CustomScrollbar/index.js';import {calculateTimeDurationSec}from'@admin-layout/timetracker-core';import {TimerActivityItem}from'../../components/TimerActivityItem/index.js';import moment from'moment';import {formatDuration}from'../../services/timeRecordService.js';import {useTimeformat,useCreatePermissions,useDeletePermissions,useSplitTimeRecordsPerDay}from'../../hooks/index.js';import {TRACKER_MODE}from'../../constants/enums.js';import*as _ from'lodash';const currentWeekRecords = (timeRecords, weekStart) => {
    return timeRecords.filter((record) => {
        return moment(record.startTime) >= moment(weekStart) && moment(record.endTime) < moment(weekStart).add(1, 'week');
    });
};
const pastWeekRecords = (timeRecords, weekStart) => {
    return timeRecords.filter((record) => {
        return moment(record.startTime) < moment(weekStart);
    });
};
const groupTimeRecords = (timeRecords, weekStart, dateFormat) => {
    if (!timeRecords)
        return [];
    timeRecords.sort((a, b) => {
        if (moment(a.endTime) < moment(b.endTime))
            return 1;
        else if (moment(a.endTime) > moment(b.endTime))
            return -1;
        else
            return 0;
    });
    let grouppedDates = {};
    for (let i = 0; i < timeRecords.length; i++) {
        const date = moment(timeRecords[i].endTime);
        const dateStr = weekStart > date ? moment(date).startOf('week').format(dateFormat) : date.format(dateFormat);
        if (!grouppedDates.hasOwnProperty(dateStr)) {
            grouppedDates[dateStr] = [];
        }
        grouppedDates[dateStr].push(timeRecords[i]);
    }
    let groupedDateArray = [];
    for (let key of Object.keys(grouppedDates)) {
        let timeArray = grouppedDates[key];
        groupedDateArray.push(timeArray);
    }
    return groupedDateArray;
};
const renderDayDateString = (date, dateFormat) => {
    if (moment(date) < moment().startOf('week')) {
        return (moment(date).startOf('week').format(dateFormat) +
            ' - ' +
            moment(date).startOf('week').add('6', 'day').format(dateFormat));
    }
    if (moment(date).format('YYYY-MM-DD') === moment().format('YYYY-MM-DD'))
        return 'Today';
    else
        return moment(date).format(dateFormat);
};
const TimerList = (props) => {
    const { isRecording, projects, tasks, weekStart, createTimeRecord, removeTimeRecord, updateTimeRecord, createTask, stopTimer, setMode, } = props;
    const [loading, setLoading] = useState(true);
    const { timeFormat, dateFormat } = useTimeformat();
    const { self: createPermit } = useCreatePermissions();
    const { self: deletePermit } = useDeletePermissions();
    const timeRecords = useSplitTimeRecordsPerDay(props.timeRecords);
    useEffect(() => {
        setTimeout(() => {
            setLoading(false);
        }, 10);
    }, []);
    const renderTotalTimeByDay = (timeRecords) => {
        let totalTime = 0;
        for (let i = 0; i < timeRecords.length; i++) {
            totalTime += calculateTimeDurationSec(timeRecords[i].startTime, timeRecords[i].endTime, timeRecords[i].pauseStatus);
        }
        return formatDuration(Math.floor(totalTime), timeFormat);
    };
    const handlePlayTimer = (timeRecord) => {
        if (isRecording)
            stopTimer();
        setMode(TRACKER_MODE.TRACK);
        const newTimeRecord = Object.assign(Object.assign({}, _.omit(timeRecord, ['id', '__typename', 'isEndDay'])), { startTime: moment(), endTime: null });
        createTimeRecord(newTimeRecord);
    };
    const TimeRecordGroup = (records, index) => {
        return (React.createElement(Card, { title: React.createElement(Row, null,
                React.createElement(Col, null, renderDayDateString(records[0].endTime, 'ddd, MMM DD')),
                React.createElement(Spacer, null),
                React.createElement(Col, null,
                    " Total time: ",
                    renderTotalTimeByDay(records))), style: { marginTop: '10px' }, bodyStyle: { padding: '0px' }, key: `timerecord-group-${index}` }, records.map((timeRecord, index) => (React.createElement(TimerActivityItem, { key: index, timeRecord: timeRecord, timeRecords: timeRecords, projects: projects, tasks: tasks, disablePlay: createPermit !== "Allow" /* Allow */, disableDelete: deletePermit !== "Allow" /* Allow */, removeTimeRecord: removeTimeRecord, updateTimeRecord: updateTimeRecord, handlePlayTimer: handlePlayTimer, createTask: createTask })))));
    };
    return useMemo(() => {
        let records = [];
        if (!loading) {
            records = timeRecords;
        }
        const currentRecords = currentWeekRecords(records, weekStart);
        return (React.createElement(CustomScrollbar, null,
            React.createElement(Spin, { spinning: loading },
                React.createElement("div", { className: "main-page__list" },
                    React.createElement("div", { className: "current-week" },
                        React.createElement(Row, { style: { padding: '0 20px' } },
                            React.createElement(Col, null, "Current Week"),
                            React.createElement(Spacer, null),
                            React.createElement(Col, null, renderTotalTimeByDay(currentRecords))),
                        groupTimeRecords(currentRecords, weekStart, dateFormat).map((dayRecords, index) => TimeRecordGroup(dayRecords, index))),
                    groupTimeRecords(pastWeekRecords(records, weekStart), weekStart, dateFormat).map((dayRecords, index) => TimeRecordGroup(dayRecords, index))))));
    }, [timeRecords, projects, tasks, weekStart, dateFormat, createPermit, deletePermit, loading, isRecording]);
};export{TimerList};//# sourceMappingURL=TimerList.js.map
