import React__default,{useState,useEffect}from'react';import {useSelector,useDispatch}from'react-redux';import moment from'moment';import {useGetDurationTimeRecordsQuery,useCreateTimeRecordMutation,useRemoveTimeRecordMutation,useUpdateTimeRecordMutation,useGetProjectsQuery}from'@admin-layout/timetracker-core/lib/generated/react-hooks';import {currentTimerSelector,SubscribeToTimeTrackerDocument,setCurrentTimerAction}from'@admin-layout/timetracker-core';import {useGetTasksQuery,useCreateTaskMutation}from'@adminide-stack/account-api-client';import {Spin,message}from'antd';import*as _ from'lodash';import {omit}from'lodash';import {useFirstWeekDay,useCreatePermissions,useDeletePermissions}from'../../hooks/index.js';import TimerActivity from'./TimerActivity.js';const TimeTrackerWrapper = (props) => {
    const userId = useSelector((state) => state.user.auth0UserId);
    const orgName = useSelector((state) => state.platform.orgName);
    const [range, setRange] = useState({ startTime: moment().startOf('month'), endTime: moment().endOf('month') });
    const { data, loading, refetch, subscribeToMore: subscribeDurationTimeRecord, } = useGetDurationTimeRecordsQuery({
        variables: { userId, startTime: range.startTime, endTime: range.endTime },
    });
    const currentTimer = useSelector(currentTimerSelector);
    // const { data: plData, refetch: plRefetch, loading: plLoading, subscribeToMore } = useGetPlayingTimeRecordQuery();
    const { data: taskData, refetch: tkRefetch, loading: tkLoading } = useGetTasksQuery();
    const [createMutation] = useCreateTimeRecordMutation();
    const [createTaskMutation] = useCreateTaskMutation();
    const [removeMutation] = useRemoveTimeRecordMutation();
    const [updateMutation] = useUpdateTimeRecordMutation();
    const { data: projectsData, loading: loadingProjects, refetch: refetchProjects } = useGetProjectsQuery();
    const { value: dowValue } = useFirstWeekDay();
    const [weekStart, setWeekStart] = useState(moment().startOf('week'));
    const { self: createPermit } = useCreatePermissions();
    const { self: deletePermit } = useDeletePermissions();
    const dispatch = useDispatch();
    useEffect(() => {
        moment.locale('en', {
            week: {
                dow: dowValue,
            },
        });
        setWeekStart(moment().startOf('week'));
    }, []);
    useEffect(() => {
        refetch();
        refetchProjects();
        tkRefetch();
    }, [orgName]);
    // create time record
    const createTimeRecord = (request) => {
        var _a;
        if (createPermit !== "Allow" /* Allow */) {
            message.warning('Permission Not Allowed');
            return;
        }
        request = Object.assign(Object.assign({}, omit(request, ['id', '__typename'])), { pauseStatus: (_a = request.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({ pausedTime, restartTime })) });
        return createMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.info('TimeRecord created');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    // create task
    const createTask = (request) => {
        if (createPermit !== "Allow" /* Allow */) {
            message.warning('Permission Not Allowed');
            return;
        }
        return createTaskMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.info('Task created');
                    cache.modify({
                        fields: {
                            getTasks() { }
                        }
                    });
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    // remove time record
    const removeTimeRecord = (recordId) => {
        if (deletePermit !== "Allow" /* Allow */) {
            message.warning('Permission Not Allowed');
            return;
        }
        removeMutation({
            variables: { recordId },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.success('TimeRecord Removed');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    // update time record
    const updateTimeRecord = (recordId, request) => {
        var _a;
        request = Object.assign(Object.assign({}, omit(request, ['id', '__typename'])), { pauseStatus: (_a = request.pauseStatus) === null || _a === void 0 ? void 0 : _a.map(({ pausedTime, restartTime }) => ({
                pausedTime,
                restartTime,
                status: "BY_USER" /* BY_USER */,
            })) });
        return updateMutation({
            variables: { recordId, request },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.success('TimeRecord Updated');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    // remove current playing time record
    const removePlayingTimeRecord = () => {
        removeMutation({
            variables: { recordId: currentTimeRecord.id },
            update: (cache, { data, errors }) => {
                if (data) {
                    message.success('TimeRecord Removed');
                }
                else {
                    message.error(errors[0].message);
                }
            },
        });
    };
    const resetTimerValues = () => {
        setIsRecording(false);
        const defaultPlayingRecord = {
            startTime: null,
            description: '',
            endTime: null,
            isBillable: false,
            projectId: '',
            taskName: '',
            isPause: false,
            isTimeout: false,
        };
        setCurrentTimeRecord(Object.assign({}, defaultPlayingRecord));
        dispatch(setCurrentTimerAction(Object.assign({}, defaultPlayingRecord)));
    };
    const getSubscriptionDurations = ({ userId, orgName, }) => {
        return {
            document: SubscribeToTimeTrackerDocument,
            variables: { userId, orgName },
            updateQuery: (prev, { subscriptionData }) => {
                var _a;
                const subscribedData = subscriptionData.data.SubscribeToTimeTracker;
                if (!subscribedData) {
                    return prev;
                }
                if (subscribedData.mutation === "TimeRecordUpdated" /* TimeRecordUpdated */ &&
                    ((_a = subscribedData.timeRecord) === null || _a === void 0 ? void 0 : _a.endTime) !== null) {
                    const index = prev.getDurationTimeRecords.findIndex((record) => (record.startTime === subscribedData.timeRecord.startTime));
                    let newData = prev.getDurationTimeRecords.slice(0);
                    if (index === -1) {
                        newData.push(Object.assign({}, subscribedData.timeRecord));
                    }
                    else {
                        newData.splice(index, 1, Object.assign({}, subscribedData.timeRecord));
                    }
                    return { getDurationTimeRecords: newData };
                }
                else if (subscribedData.mutation === "TimeRecordDeleted" /* TimeRecordDeleted */) {
                    const index = prev.getDurationTimeRecords.findIndex((record) => (record.id === subscribedData.timeRecord.id));
                    let newData = prev.getDurationTimeRecords.slice(0);
                    if (index !== -1) {
                        newData.splice(index, 1);
                    }
                    return { getDurationTimeRecords: newData };
                }
            }
        };
    };
    useEffect(() => {
        const unsubscribe = subscribeDurationTimeRecord(getSubscriptionDurations({ userId, orgName }));
        return () => unsubscribe();
    }, [orgName, userId, subscribeDurationTimeRecord]);
    const [currentTimeRecord, setCurrentTimeRecord] = useState({
        startTime: null,
        endTime: null,
        isBillable: false,
        description: '',
        projectId: '',
        taskName: '',
        isPause: false,
        isTimeout: false,
    });
    const [isRecording, setIsRecording] = useState(false);
    useEffect(() => {
        if (currentTimer && (currentTimer.startTime === null || currentTimer.endTime !== null)) {
            if (isRecording) {
                resetTimerValues();
            }
        }
        else {
            // const passDur = moment().valueOf() - moment(playingRecord.startTime).valueOf();
            setCurrentTimeRecord(currentTimer);
            setIsRecording(true);
        }
    }, [currentTimer]);
    return (React__default.createElement(Spin, { spinning: !data || loading },
        React__default.createElement(TimerActivity, Object.assign({}, props, { weekStart: weekStart, range: range, projects: _.get(projectsData, 'getProjects', []), createTimeRecord: createTimeRecord, removeTimeRecord: removeTimeRecord, removePlayingTimeRecord: removePlayingTimeRecord, updateTimeRecord: updateTimeRecord, timeRecords: _.get(data, 'getDurationTimeRecords', []), tasks: _.get(taskData, 'getTasks', []), setIsRecording: setIsRecording, setCurrentTimeRecord: setCurrentTimeRecord, currentTimeRecord: currentTimeRecord, isRecording: isRecording, setRange: setRange, createTask: createTask }))));
};export default TimeTrackerWrapper;//# sourceMappingURL=index.js.map
