import React__default,{useEffect}from'react';import {Form,Modal,Avatar,Select,Row,Col,DatePicker,Checkbox,Button,Popconfirm,TimePicker,Input}from'antd';import {UserOutlined,DeleteOutlined}from'@ant-design/icons';import {useFela}from'react-fela';import moment from'moment';import Spacer from'../../../components/Spacer/index.js';import {useTimeformat}from'../../../hooks/index.js';const { RangePicker } = TimePicker;
const { TextArea } = Input;
const TimesheetModal = ({ userId, event, isShowModal, members, projects, loading, handleAddTimeRecordEvent, handleUpdateTimeRecordEvent, handleRemoveTimeRecordEvent, handleCloseTimeModal, }) => {
    const { css } = useFela();
    const [form] = Form.useForm();
    const { timeFormat } = useTimeformat();
    useEffect(() => {
        form.setFieldsValue({
            userId: (event === null || event === void 0 ? void 0 : event.userId) || '',
            projectId: (event === null || event === void 0 ? void 0 : event.projectId) || '',
            isBillable: (event === null || event === void 0 ? void 0 : event.isBillable) || false,
            timeRange: [(event === null || event === void 0 ? void 0 : event.startTime) || moment(), (event === null || event === void 0 ? void 0 : event.endTime) || moment()],
            date: (event === null || event === void 0 ? void 0 : event.startTime) || moment(),
            description: (event === null || event === void 0 ? void 0 : event.description) || '',
        });
    }, [event]);
    const onFinish = (values) => {
        let range = {};
        if (event.id === undefined || !event.isSplit) {
            range = {
                startTime: moment(values.date.format('YYYY-MM-DD') + ' ' + values.timeRange[0].format('HH:mm:ss')).toDate(),
                endTime: moment(values.date.format('YYYY-MM-DD') + ' ' + values.timeRange[1].format('HH:mm:ss')).toDate(),
            };
        }
        const request = Object.assign(Object.assign({}, range), { userId: values.userId, projectId: values.projectId, isBillable: values.isBillable, description: values.description });
        if (event.id === undefined)
            handleAddTimeRecordEvent(request);
        else
            handleUpdateTimeRecordEvent(event.id, request);
        handleCloseTimeModal();
    };
    const resetModal = () => {
        form.resetFields();
    };
    const generateUserName = () => {
        const user = members.find((m) => m.userId === userId);
        if (user !== undefined)
            return user.name;
        return '';
    };
    return (React__default.createElement(Modal, { title: event.id === undefined ? 'Add time record' : 'Edit time record', visible: isShowModal, onCancel: handleCloseTimeModal, footer: false },
        React__default.createElement(Form, { labelCol: { span: 24 }, wrapperCol: { span: 24 }, layout: "vertical", onFinish: onFinish, className: css(stylesheet.form), form: form },
            React__default.createElement("div", { style: { margin: '15px 0px' } },
                React__default.createElement(Avatar, { style: { backgroundColor: '#3174ad' }, icon: React__default.createElement(UserOutlined, null) }),
                React__default.createElement("span", { style: { marginLeft: '10px' } }, generateUserName())),
            React__default.createElement(Form.Item, { label: "User", name: "userId", rules: [{ required: true, message: 'Required field' }] },
                React__default.createElement(Select, { disabled: !!event.timesheetId }, members.map((member) => {
                    return (React__default.createElement(Select.Option, { value: member.userId, key: member.userId }, member.name));
                }))),
            React__default.createElement(Form.Item, { label: "Projects", name: "projectId", rules: [{ required: true, message: 'Required field' }] },
                React__default.createElement(Select, { disabled: !!event.timesheetId }, projects.map((res) => {
                    return (React__default.createElement(Select.Option, { value: res.id, key: res.id }, res.name));
                }))),
            React__default.createElement(Form.Item, { label: "Task", name: "taskId" },
                React__default.createElement(Select, { disabled: (event === null || event === void 0 ? void 0 : event.projectId) === '' || !!!(event === null || event === void 0 ? void 0 : event.projectId) || !!event.timesheetId }, [].map((task) => {
                    return (React__default.createElement(Select.Option, { value: task.id, key: task.id }, task.name));
                }))),
            React__default.createElement(Row, { gutter: 10 },
                React__default.createElement(Col, null,
                    React__default.createElement(Form.Item, { label: "Pick a date", name: "date", rules: [{ required: true, message: 'Required field' }] },
                        React__default.createElement(DatePicker, { disabled: event.isSplit || !!event.timesheetId }))),
                React__default.createElement(Col, null,
                    React__default.createElement(Form.Item, { label: "Select time range", name: "timeRange", rules: [{ required: true, message: 'Required field' }] },
                        React__default.createElement(RangePicker, { format: timeFormat, bordered: false, disabled: event.isSplit || !!event.timesheetId })))),
            React__default.createElement(Form.Item, { label: "Billable", name: "isBillable", valuePropName: "checked" },
                React__default.createElement(Checkbox, { disabled: !!event.timesheetId })),
            React__default.createElement(Form.Item, { label: "Tags", name: "tags" },
                React__default.createElement(Select, { mode: "tags", disabled: !!event.timesheetId })),
            React__default.createElement(Form.Item, { label: "Description", name: "description" },
                React__default.createElement(TextArea, { disabled: !!event.timesheetId, value: event === null || event === void 0 ? void 0 : event.description })),
            React__default.createElement(Form.Item, null,
                React__default.createElement(Row, { className: "footer" },
                    React__default.createElement(Button, { htmlType: "button", onClick: resetModal, disabled: !!event.timesheetId }, "Reset"),
                    "\u00A0",
                    (event !== -1 && handleRemoveTimeRecordEvent) ? (React__default.createElement(Popconfirm, { title: "Are you sure to remove event", okText: "OK", cancelText: "Cancel", onConfirm: handleRemoveTimeRecordEvent },
                        React__default.createElement(Button, { type: "primary", htmlType: "button", loading: loading, icon: React__default.createElement(DeleteOutlined, null), disabled: !!event.timesheetId, danger: true }, "Remove"))) : (''),
                    React__default.createElement(Spacer, null),
                    React__default.createElement(Button, { type: "primary", htmlType: "submit", loading: loading, disabled: !!event.timesheetId }, "Submit"))))));
};
const stylesheet = {
    form: (props) => ({
        display: 'block',
        '& .footer': {
            display: 'flex',
            flexDirection: 'row',
        },
    }),
};export default TimesheetModal;//# sourceMappingURL=TimesheetModal.js.map
