import React__default,{useState,useEffect}from'react';import SelectableCalendar from'./Timesheet.js';import {useCreateTimeRecordMutation,useUpdateTimeRecordMutation,useRemoveTimeRecordMutation,useGetDurationTimeRecordsQuery}from'@admin-layout/timetracker-core/lib/generated/react-hooks';import {Spin,message}from'antd';import moment from'moment';import {useSelector}from'react-redux';import {useSplitTimeRecordsPerDay}from'../../../hooks/index.js';const defaultRecord = {
    id: undefined,
    projectId: undefined,
    userId: undefined,
    startTime: undefined,
    endTime: undefined,
    isBillable: false,
    tags: [],
    description: undefined,
};
const TimesheetCalendar = ({ projects, members, selectedUser, selectedProject, isShowTimeModal, setIsShowTimeModal, localizer, weekStart, setPathWeekStart, }) => {
    const [selectedEvent, setSelectedEvent] = useState(defaultRecord);
    const [selectedTask, setSelectedTask] = useState('');
    const [selectableTasks, setSelectableTasks] = useState([]);
    const [addMutation, { loading: loadingAdd }] = useCreateTimeRecordMutation();
    const [updateMutation, { loading: loadingUpdate }] = useUpdateTimeRecordMutation();
    const [removeMutation, { loading: loadingRemove }] = useRemoveTimeRecordMutation();
    // const [form] = Form.useForm();
    const userId = useSelector((state) => state.user.auth0UserId);
    // create event handler
    const handleAddTimeRecordEvent = (request) => {
        addMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('A new event has been created!');
                    setIsShowTimeModal(false);
                }
                else {
                    console.log(errors[0].message);
                    message.error('Event creation failed!');
                }
            },
        });
    };
    // update event handler
    const handleUpdateTimeRecordEvent = (recordId, request) => {
        updateMutation({
            variables: { recordId, request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('A new event has been updated!');
                    setIsShowTimeModal(false);
                }
                else {
                    console.log(errors[0].message);
                    message.error('Event update failed!');
                }
            },
        });
    };
    // remove event handler
    const handleRemoveTimeRecordEvent = () => {
        removeMutation({
            variables: { recordId: selectedEvent.id },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('Event has removed');
                    setIsShowTimeModal(false);
                }
                else {
                    console.log(errors[0].message);
                }
            }
        });
    };
    // event selection handler
    const handleSelectEvent = (event) => {
        setSelectedEvent({
            id: event.id,
            userId: event.userId,
            projectId: event.projectId,
            startTime: moment(event.startTime),
            endTime: moment(event.endTime),
            isBillable: event.isBillable || false,
            tags: event.tags || [],
            description: event.description,
            isSplit: event.isSplit,
        });
        setIsShowTimeModal(true);
    };
    // new slot selection handler
    const handleSelectSlot = ({ start, end }) => {
        setSelectedEvent(Object.assign(Object.assign({}, selectedEvent), { startTime: moment(start), endTime: moment(end) }));
        setIsShowTimeModal(true);
    };
    const [timeRecords, setTimeRecords] = useState([]);
    const { data: { getDurationTimeRecords } = {}, loading } = useGetDurationTimeRecordsQuery({
        variables: {
            startTime: weekStart,
            endTime: moment(weekStart).add(1, 'week'),
        },
    });
    const records = useSplitTimeRecordsPerDay(getDurationTimeRecords);
    useEffect(() => {
        const filterEvent = filterEvents(records || []);
        const recordsWithSplit = filterEvent.map((record) => {
            const isSplit = filterEvent.filter((data) => data.id === record.id).length > 1;
            return (Object.assign(Object.assign({}, record), { isSplit, title: isSplit ? 'Can not editable due to split record' : null }));
        });
        setTimeRecords(recordsWithSplit);
    }, [records, members, selectedUser, selectedProject]);
    const filterEvents = (events) => {
        if (!events)
            return [];
        let newEvents = [];
        events
            .filter((ev) => {
            return ((ev.userId === selectedUser || selectedUser === '__all') &&
                (ev.projectId === selectedProject || selectedProject === '__all') &&
                members.findIndex((mem) => mem.userId === ev.userId) > -1);
        })
            .map((ev) => (Object.assign(Object.assign({}, ev), { startTime: moment(ev.startTime).toDate(), endTime: moment(ev.endTime).toDate() })))
            .forEach((record) => {
            if (!record.pauseStatus.length) {
                newEvents.push(Object.assign(Object.assign({}, record), { startTime: record.startTime, endTime: record.endTime }));
            }
            else {
                const { pauseStatus } = record;
                for (let i = 0; i < pauseStatus.length; i = i + 1) {
                    if (i === 0) {
                        newEvents.push(Object.assign(Object.assign({}, record), { startTime: record.startTime, endTime: moment(pauseStatus[i].pausedTime).toDate() }));
                        if (pauseStatus[i].restartTime && pauseStatus.length === 1) {
                            newEvents.push(Object.assign(Object.assign({}, record), { startTime: moment(pauseStatus[i].restartTime).toDate(), endTime: record.endTime }));
                        }
                    }
                    else if (i === (pauseStatus[i].length - 1)) {
                        newEvents.push(Object.assign(Object.assign({}, record), { startTime: moment(pauseStatus[i - 1].restartTime).toDate(), endTime: moment(pauseStatus[i].pausedTime).toDate() }));
                        if (pauseStatus[i].restartTime) {
                            newEvents.push(Object.assign(Object.assign({}, record), { startTime: moment(pauseStatus[i].restartTime).toDate(), endTime: record.endTime }));
                        }
                    }
                    else {
                        newEvents.push(Object.assign(Object.assign({}, record), { startTime: moment(pauseStatus[i - 1].restartTime).toDate(), endTime: moment(pauseStatus[i].pausedTime).toDate() }));
                    }
                }
            }
        });
        return newEvents;
    };
    const handleChangeTask = (value) => {
        setSelectedTask(value);
    };
    const handleOpenNewTimeModal = () => {
        setSelectedEvent(defaultRecord);
        setIsShowTimeModal(true);
    };
    const handleCloseTimeModal = () => {
        setSelectedEvent(defaultRecord);
        setIsShowTimeModal(false);
    };
    return (React__default.createElement(Spin, { spinning: !timeRecords || loading },
        React__default.createElement(SelectableCalendar, { userId: userId, events: timeRecords, 
            // form={form}
            loading: loadingAdd || loadingUpdate || loadingRemove, projects: projects, tasks: selectableTasks, members: members, weekStart: weekStart, isShowModal: isShowTimeModal, selectedUser: selectedUser, selectedProject: selectedProject, selectedTask: selectedTask, selectedEvent: selectedEvent, setSelectedEvent: setSelectedEvent, localizer: localizer, handleAddTimeRecordEvent: handleAddTimeRecordEvent, handleUpdateTimeRecordEvent: handleUpdateTimeRecordEvent, handleRemoveTimeRecordEvent: handleRemoveTimeRecordEvent, handleOpenNewTimeModal: handleOpenNewTimeModal, handleCloseTimeModal: handleCloseTimeModal, handleSelectSlot: handleSelectSlot, handleSelectEvent: handleSelectEvent, handleChangeTask: handleChangeTask, setPathWeekStart: setPathWeekStart })));
};export default TimesheetCalendar;//# sourceMappingURL=index.js.map
