import React__default,{useState,useEffect}from'react';import ReactTable from'react-table';import {useFela}from'react-fela';import {Avatar}from'antd';import moment from'moment';import {COMMAND_ACTION_TYPES}from'@adminide-stack/extension-api';import {WithPermissionEnhanced}from'@adminide-stack/platform-browser/lib/ant-module';import {IPreDefineAccountPermissions}from'@adminide-stack/core';import {formatDuration}from'../../../services/timeRecordService.js';import {TableActions}from'./TableAction.js';const generateAvatarStr = (str) => str === null || str === void 0 ? void 0 : str.split(' ').map(word => word[0]).join('').toUpperCase();
const ProjectRecord = (props) => {
    const { project, task, client } = props.value;
    const { css } = useFela();
    return (React__default.createElement("div", { className: css(StyleSheet.flex) },
        React__default.createElement("div", null,
            React__default.createElement(Avatar, { className: css(StyleSheet.avtar) + " mr-2" }, generateAvatarStr(project))),
        React__default.createElement("div", { className: css(StyleSheet.ml7) },
            React__default.createElement("div", { className: css(StyleSheet.bold) }, project),
            React__default.createElement("div", { className: css(StyleSheet.client) }, client || ''),
            React__default.createElement("div", { className: css(StyleSheet.bold) }, task || 'No to-do'))));
};
const Activity = (props) => {
    return (React__default.createElement("div", null, props.value));
};
const Duration = (props) => {
    const { css } = useFela();
    return (React__default.createElement("div", null,
        React__default.createElement("span", { className: css(StyleSheet.fontColor) }, props.value),
        React__default.createElement("span", { className: css(StyleSheet.bold, StyleSheet.ml7) }, "$")));
};
const TimeRecord = (props) => {
    const { css } = useFela();
    return (React__default.createElement("div", { className: css(StyleSheet.fontColor) }, props.value));
};
const ActionsComponent = (props) => {
    const { dispatch, record, members, projects, contributions: { menus = {}, actions = [] } } = props.value;
    const [eleProps, setEleProps] = useState({});
    useEffect(() => {
        setEleProps({
            event: Object.assign(Object.assign({}, record), { startTime: moment(record.startTime), endTime: moment(record.endTime) }),
            members,
            projects,
        });
    }, [JSON.stringify(record), members, projects]);
    const menu = menus['dailyDetails/menu'];
    const menuActions = menu && menu.map((item) => {
        const action = actions.find((data) => data.id === item.action);
        const commandArguments = action.commandArguments && action.commandArguments.map((arg) => arg.startsWith('${') ? eval(arg.substring(2, arg.length - 1)) : arg);
        let danger = false;
        let messageValue = '';
        let type = 'primary';
        if (action.id === 'removeTimeRecord') {
            danger = true;
            type = '';
            messageValue = 'Are you sure to remove record?';
        }
        return ({
            type,
            className: 'mr-2',
            name: action && action.title,
            danger,
            message: messageValue,
            action: () => {
                dispatch({
                    type: COMMAND_ACTION_TYPES.EXECUTE_COMMAND,
                    payload: {
                        command: action && action.command,
                        arguments: commandArguments,
                    },
                });
            },
        });
    });
    return (React__default.createElement(WithPermissionEnhanced, { permissionKeys: [
            IPreDefineAccountPermissions.manageTeams,
            IPreDefineAccountPermissions.editTeams,
        ] },
        React__default.createElement(TableActions, { actions: menuActions })));
};
const DailyDetailsTable = (props) => {
    const { css } = useFela();
    const [columns, setColumns] = useState([]);
    const [data, setData] = useState([]);
    useEffect(() => {
        const { contributions: { tables = {} } } = props;
        const table = tables && tables['dailyDetails/table'];
        const columnsArray = table && table.map((column) => {
            return ({
                id: column.id,
                accessor: column.accessor,
                Header: () => React__default.createElement("div", { className: css(StyleSheet.header) }, column.title),
                Cell: column.cell,
                minWidth: column.width || 100,
                headerClassName: css(StyleSheet.headerClass),
                className: css(StyleSheet.tableData),
            });
        });
        const tableData = props.data && props.data.map((d) => {
            var _a, _b;
            const { members, projects } = props;
            const project = {
                project: ((_a = projects.find((project) => project.id === d.projectId)) === null || _a === void 0 ? void 0 : _a.name) || 'Unknown',
                task: d.taskName,
                client: (_b = props.clients.find((client) => client.id === d.clientId)) === null || _b === void 0 ? void 0 : _b.displayName,
            };
            return ({
                project: project,
                activity: d.activity + '%',
                duration: formatDuration(d.duration),
                time: `${moment(d.startTime).format('h:mm a')} - ${moment(d.endTime).format('h:mm a')}`,
                actions: { dispatch: props.dispatch, contributions: props.contributions, record: d, members, projects },
            });
        });
        if (columnsArray) {
            setData(tableData || []);
            setColumns(columnsArray);
        }
    }, [props.contributions, props.data, props.projects, props.clients]);
    return data ? (React__default.createElement("div", { className: css(StyleSheet.container) },
        React__default.createElement(ReactTable, { data: data || [], columns: columns || [], sortable: false, showPagination: !!data.length, showPageSizeOptions: false, pageSize: data.length > 5 ? 5 : data.length, resolveData: (resData) => resData.map(row => row), noDataText: 'No Data' }))) : null;
};
const StyleSheet = {
    container: () => ({
        '& .rt-thead': {
            backgroundColor: '#f5f5f5',
        },
        '& .rt-noData': {
            position: 'relative',
            margin: '32px 0',
            marginBottom: '-20px',
            textAlign: 'center',
        },
    }),
    headerClass: () => ({
        margin: '0 5px',
        textAlign: 'left',
    }),
    header: () => ({
        color: '#000000d9',
        padding: '14px 0',
        fontSize: '14px',
        fontWeight: '500',
    }),
    tableData: () => ({
        color: '#000000d9',
        margin: '10px 5px',
        fontSize: '14px',
    }),
    flex: () => ({
        display: 'flex',
    }),
    ml7: () => ({
        marginLeft: '7px',
    }),
    bold: () => ({
        fontWeight: 500,
    }),
    fontColor: () => ({
        color: '#1890ff',
    }),
    client: () => ({
        color: '#787e84',
        fontSize: '13px',
    }),
    avtar: () => ({
        backgroundColor: '#096dd9',
    }),
};export{ActionsComponent,Activity,DailyDetailsTable,Duration,ProjectRecord,TimeRecord};//# sourceMappingURL=DailyDetailsTable.js.map
