import React__default,{useState,useEffect}from'react';import {Menu,Modal,Button,Row,Col,Tag,Popconfirm,Dropdown,Typography}from'antd';import moment from'moment';import {useFela}from'react-fela';import classNames from'classnames';import {calculateTimeDurationSec}from'@admin-layout/timetracker-core';import {TimesheetInput}from'../../../components/TimesheetInput/index.js';import {CloseOutlined,PlusOutlined}from'@ant-design/icons';import {formatDuration}from'../../../services/timeRecordService.js';import*as _ from'lodash';import {useTimeformat}from'../../../hooks/index.js';import {ExportExcel}from'./ExportExcel.js';import {DailyDetailsModal}from'./DailyDetailsModal.js';const { Title } = Typography;
const calcDuration = (records) => {
    return records.reduce((duration, record) => Math.abs(calculateTimeDurationSec(record.startTime, record.endTime, record.pauseStatus)) + duration, 0);
};
var TIMESHEET_STATE;
(function (TIMESHEET_STATE) {
    TIMESHEET_STATE["APPROVED"] = "Approved";
    TIMESHEET_STATE["PENDING"] = "Pending for approval";
})(TIMESHEET_STATE || (TIMESHEET_STATE = {}));
const TabularCalendar = ({ weekStart, records, projects, timesheets, selectedUser, projectsApproval, projectsMap, members, handleRemoveDuration, updateTimeRecord, createTimeRecord, createTimesheet, }) => {
    const { css } = useFela();
    const [newRows, setNewRows] = useState([]);
    const [selectedDay, setSelectedDay] = useState(null);
    const [visible, setVisible] = useState(false);
    const [showApprovalModal, setShowApprovalModal] = useState(false);
    const { timeFormat, dateFormat } = useTimeformat();
    const { approvals, unApprovals } = projectsApproval;
    useEffect(() => {
        const rows = newRows.filter((pId) => unApprovals.findIndex((upId) => upId === pId) === -1);
        setNewRows(rows);
    }, [unApprovals]);
    const getProjectTotalDuration = (projectId, approved) => {
        return calcDuration(records.filter((r) => r.projectId === projectId &&
            (approved ? !!r.timesheetId : !r.timesheetId) &&
            moment(r.startTime) >= moment(weekStart) &&
            moment(r.endTime) <= moment(weekStart).add(1, 'week')));
    };
    const getDayTotalDuration = (curDay) => {
        const dayStr = moment(curDay).format(dateFormat);
        return calcDuration(records
            .filter((r) => moment(r.startTime).format(dateFormat) === dayStr)
            .filter((r) => projects.findIndex((p) => p.id === r.projectId) !== -1));
    };
    const getTotalDuration = () => {
        return calcDuration(records.filter((r) => moment(r.startTime) >= moment(weekStart) && moment(r.endTime) <= moment(weekStart).add(1, 'week')));
    };
    const handleSelectNewProject = (projectId) => {
        setNewRows([...newRows, projectId]);
    };
    const handleRemoveNewRow = (rowId) => {
        setNewRows(newRows.filter((pId) => pId !== rowId));
    };
    const handleSubmitApproval = (event) => {
        event.preventDefault();
        const approvalRequest = {
            userId: selectedUser,
            startDate: moment(weekStart),
            endDate: moment(weekStart).add(1, 'week'),
            state: "SUBMITTED" /* SUBMITTED */,
            submittedOn: moment(),
            timeRecordsId: _.uniq(records.filter(record => !record.timesheetId)
                .map((record) => record.id)),
        };
        createTimesheet(approvalRequest);
        setShowApprovalModal(false);
    };
    const openSubmitApproval = (event) => {
        event.preventDefault();
        setShowApprovalModal(true);
    };
    const handleCloseApproval = () => {
        setShowApprovalModal(false);
    };
    const selectableProjects = () => {
        // selectable projects should not be involved to unApprovedProjects() or newRows
        return projects.filter((p) => unApprovals.findIndex((upId) => upId === p.id) === -1 && newRows.findIndex((pId) => pId === p.id) === -1);
    };
    const projectDropdownMenus = (React__default.createElement(Menu /*className={css(styles.projectDown)}*/, null, selectableProjects().map((pr) => {
        return (React__default.createElement(Menu.Item, { key: pr.id, onClick: () => handleSelectNewProject(pr.id) }, pr.name));
    })));
    const openDetailsModal = (day) => {
        setSelectedDay(day);
        setVisible(true);
    };
    const closeDetailsModal = () => {
        setSelectedDay(null);
        setVisible(false);
    };
    const HeaderRows = () => {
        return (React__default.createElement("tr", null,
            React__default.createElement("th", null, " ProjectName "),
            React__default.createElement(DailyDetailsModal, { visible: visible, onClose: closeDetailsModal, selectedDay: selectedDay, userId: selectedUser, projects: projects, members: members, records: records, weekStart: weekStart }),
            Array(7)
                .fill(0)
                .map((val, index) => {
                const curDay = moment(weekStart).add(index, 'day');
                const styledClass = ((selectedDay === null || selectedDay === void 0 ? void 0 : selectedDay.format(dateFormat)) === (curDay === null || curDay === void 0 ? void 0 : curDay.format(dateFormat))) ?
                    css(styles.dateTh, styles.selectedDay) :
                    css(styles.dateTh);
                return (React__default.createElement("th", { key: index, className: styledClass, onClick: () => openDetailsModal(curDay) },
                    React__default.createElement("div", { className: css(styles.dateHeader) },
                        React__default.createElement("div", { className: classNames('day') },
                            React__default.createElement("span", null, curDay.format('DD'))),
                        React__default.createElement("div", { className: "extra" },
                            React__default.createElement("div", { className: "week" },
                                React__default.createElement("span", null, curDay.format('ddd'))),
                            React__default.createElement("div", { className: "month" },
                                React__default.createElement("span", null, curDay.format('MMM')))))));
            }),
            React__default.createElement("th", null, " Total "),
            React__default.createElement("th", null, " Action ")));
    };
    const UnApprovedRows = () => {
        return unApprovals.map((pId) => {
            const project = projectsMap.get(pId);
            return (React__default.createElement("tr", { key: project.id },
                React__default.createElement("td", null,
                    " ",
                    project.name),
                Array(7)
                    .fill(0)
                    .map((val, index) => {
                    const curDay = moment(weekStart).add(index, 'day');
                    const curDayRecords = records.filter((r) => r.projectId === project.id &&
                        moment(r.startTime).format(dateFormat) === curDay.format(dateFormat) &&
                        !r.timesheetId);
                    return (React__default.createElement("td", { key: curDay.format(dateFormat) },
                        React__default.createElement(TimesheetInput, { dateStr: curDay.format(dateFormat), projectId: project.id, records: curDayRecords, userId: selectedUser, createTimeRecord: createTimeRecord, updateTimeRecord: updateTimeRecord, projects: projects, projectTitle: project.name })));
                }),
                React__default.createElement("td", null,
                    " ",
                    formatDuration(getProjectTotalDuration(project.id, false), timeFormat)),
                React__default.createElement("td", null,
                    React__default.createElement(Popconfirm, { title: "Are you sure to remove event", okText: "OK", cancelText: "Cancel", onConfirm: () => handleRemoveDuration(project.id) },
                        React__default.createElement(Button, { icon: React__default.createElement(CloseOutlined, null) })))));
        });
    };
    const ApprovedRows = () => {
        return approvals.map((pId) => {
            const project = projectsMap.get(pId);
            return (React__default.createElement("tr", { key: project.id },
                React__default.createElement("td", null,
                    " ",
                    project.name),
                Array(7)
                    .fill(0)
                    .map((val, index) => {
                    const curDay = moment(weekStart).add(index, 'day');
                    const curDayRecords = records.filter((r) => r.projectId === project.id &&
                        moment(r.startTime).format(dateFormat) === curDay.format(dateFormat) &&
                        !!r.timesheetId);
                    return (React__default.createElement("td", { key: curDay.format(dateFormat) },
                        React__default.createElement(TimesheetInput, { dateStr: curDay.format(dateFormat), projectId: project.id, userId: selectedUser, records: curDayRecords, createTimeRecord: createTimeRecord, updateTimeRecord: updateTimeRecord, projects: projects, projectTitle: project.name, disabled: true })));
                }),
                React__default.createElement("td", null,
                    " ",
                    formatDuration(getProjectTotalDuration(project.id, true), timeFormat)),
                React__default.createElement("td", null,
                    React__default.createElement(Popconfirm, { title: "Are you sure to remove event", okText: "OK", cancelText: "Cancel", onConfirm: () => handleRemoveDuration(project.id), disabled: true },
                        React__default.createElement(Button, { icon: React__default.createElement(CloseOutlined, null), disabled: true })))));
        });
    };
    const NewRows = () => {
        return newRows.map((pId) => {
            const project = projects.find((p) => p.id === pId);
            return (React__default.createElement("tr", { key: pId },
                React__default.createElement("td", null,
                    " ",
                    project.name),
                Array(7)
                    .fill(0)
                    .map((val, index) => {
                    const curDay = moment(weekStart).add(index, 'day');
                    return (React__default.createElement("td", { key: curDay.format(dateFormat) },
                        React__default.createElement(TimesheetInput, { dateStr: curDay.format(dateFormat), projectId: pId, userId: selectedUser, createTimeRecord: createTimeRecord, updateTimeRecord: updateTimeRecord, projects: projects })));
                }),
                React__default.createElement("td", null, "00:00:00"),
                React__default.createElement("td", null,
                    React__default.createElement(Popconfirm, { title: "Are you sure to remove event", okText: "OK", cancelText: "Cancel", onConfirm: () => handleRemoveNewRow(pId) },
                        React__default.createElement(Button, { icon: React__default.createElement(CloseOutlined, null) })))));
        });
    };
    const ProjectSelectionRow = () => {
        return (React__default.createElement("tr", null,
            React__default.createElement("td", null,
                React__default.createElement(Dropdown, { overlay: projectDropdownMenus, trigger: ['click'], disabled: projects.length === 0 },
                    React__default.createElement(Button, { icon: React__default.createElement(PlusOutlined, null), disabled: projects.length === 0 }, "Select Project"))),
            Array(9)
                .fill(0)
                .map((val, index) => (React__default.createElement(EmptyCell, { key: index })))));
    };
    const TotalReportRow = () => {
        return (React__default.createElement("tr", { className: "total" },
            React__default.createElement("td", null, " Total "),
            Array(7)
                .fill(0)
                .map((val, index) => {
                const curDay = moment(weekStart).add(index, 'day');
                return React__default.createElement("td", { key: index },
                    formatDuration(getDayTotalDuration(curDay), timeFormat),
                    " ");
            }),
            React__default.createElement("td", null,
                formatDuration(getTotalDuration(), timeFormat),
                " "),
            React__default.createElement("td", null)));
    };
    const getUsername = () => {
        const member = members.find((mem) => mem.userId === selectedUser);
        if (member)
            return member.name;
        return '';
    };
    const getTimesheetState = () => {
        if (timesheets.some((sheet) => sheet.state === "SUBMITTED" /* SUBMITTED */))
            return "SUBMITTED" /* SUBMITTED */;
        if (timesheets.some((sheet) => sheet.state === "APPROVED" /* APPROVED */) && unApprovals.length === 0)
            return "APPROVED" /* APPROVED */;
        else
            return undefined;
    };
    console.log('Tabular calendar-', selectedDay === null || selectedDay === void 0 ? void 0 : selectedDay.format());
    return (React__default.createElement("div", { className: css(styles.root) },
        React__default.createElement(Modal, { title: "Submit Week for Approval", visible: showApprovalModal, onCancel: handleCloseApproval, className: css(styles.modal), footer: [
                React__default.createElement(Button, { key: "back", onClick: handleCloseApproval }, "Close"),
                React__default.createElement(Button, { key: "submit", type: "primary", onClick: handleSubmitApproval }, "Submit"),
            ] },
            React__default.createElement("p", null,
                "Ready to submit from ",
                moment(weekStart).format('MMM DD'),
                " -",
                moment(weekStart).add(6, 'day').format('MMM DD'),
                "\u00A0 approval?")),
        React__default.createElement(Row, { className: css(styles.excel) },
            React__default.createElement(ExportExcel, { weekStart: weekStart, records: records, projectsMap: projectsMap, approvals: approvals, unApprovals: unApprovals, getProjectTotalDuration: getProjectTotalDuration })),
        React__default.createElement(Row, { className: "table-header" },
            React__default.createElement(Col, { xs: 24, md: 8 },
                React__default.createElement(Title, { level: 5 }, getUsername())),
            getTimesheetState() ? (React__default.createElement(Col, { xs: 24, md: 8 },
                React__default.createElement(Tag, { color: getTimesheetState() === "SUBMITTED" /* SUBMITTED */ ? 'orange' : 'green' }, getTimesheetState() === "SUBMITTED" /* SUBMITTED */ ? 'Pending for approval' : 'Approved'))) : null),
        React__default.createElement("table", { className: css(styles.calendarTable) },
            React__default.createElement("thead", null, HeaderRows()),
            React__default.createElement("tbody", null,
                ApprovedRows(),
                UnApprovedRows(),
                NewRows(),
                ProjectSelectionRow(),
                TotalReportRow())),
        React__default.createElement(Row, { className: "table-footer" },
            getTimesheetState() === "SUBMITTED" /* SUBMITTED */ ? (React__default.createElement("p", null,
                "You can still add time while time sheet is ",
                React__default.createElement("b", null, " Pending for approval"))) : null,
            React__default.createElement("div", { className: "spacer" }),
            getTimesheetState() === undefined ? (React__default.createElement(Button, { type: "primary", onClick: openSubmitApproval }, "Submit For Approval")) : null)));
};
const styles = {
    root: (props) => ({
        display: 'block',
        '& .table-footer': {
            display: 'flex',
            flexDirection: 'row',
            paddingTop: '10px',
            paddingBottom: '10px',
        },
        '& .spacer': {
            flexGrow: '1',
        },
        '& .flex-row': {
            display: 'flex',
            flexDirection: 'row',
        },
    }),
    modal: (props) => ({
        display: 'inherited',
        '& .flex-row': {
            display: 'flex',
            flexDirection: 'row',
        },
        '& .spacer': {
            flexGrow: 1,
        },
    }),
    dateHeader: (props) => ({
        display: 'flex',
        flexDirection: 'row',
        justifyContent: 'center',
        alignItems: 'center',
        '@media (max-width: 768px)': {
            flexDirection: 'column',
        },
        '& .day': {
            fontSize: '3em',
            fontWeight: '500',
            '@media (max-width: 768px)': {
                fontSize: '14px',
            },
        },
        '& .extra': {
            display: 'flex',
            flexDirection: 'column',
            '& .week': {
                fontSize: '1em',
                '@media (max-width: 768px)': {
                    fontSize: '12px',
                },
            },
            '& .month': {
                fontSize: '1em',
                '@media (max-width: 768px)': {
                    fontSize: '12px',
                },
                color: '#bbb',
            },
        },
        '& .today': {
            color: '#1890ff',
        },
    }),
    dateTh: () => ({
        width: 'auto',
        ':hover': {
            backgroundColor: '#bebebecc',
            boxShadow: '2px -1px 5px 0px #9d9999',
            '& .month': {
                color: '#8a8484',
            },
        }
    }),
    selectedDay: () => ({
        backgroundColor: '#bebebecc',
        boxShadow: '2px -1px 5px 0px #9d9999',
        '& .month': {
            color: '#8a8484 !important',
        }
    }),
    calendarTable: (props) => ({
        width: '100%',
        background: 'white',
        border: '1px solid #bbb',
        '& .spacer': {
            flexGrow: 1,
        },
        '& thead': {
            backgroundColor: 'rgba(0, 0, 0, 0.1)',
            textAlign: 'center',
        },
        '& tbody > tr > td:first-child': {
            paddingLeft: '10px',
            paddingRight: '10px',
        },
        '& .total': {
            background: 'rgba(0,0,0,.1)',
            fontWeight: 'bold',
        },
        '& .total > td': {
            paddingTop: '20px',
            paddingBottom: '20px',
        },
    }),
    excel: () => ({
        marginTop: '0px',
        '> span': {
            marginLeft: 'auto',
        },
    }),
};
const EmptyCell = () => {
    return React__default.createElement("td", null);
};export{TabularCalendar};//# sourceMappingURL=TabularCalendar.js.map
