import React__default,{useState,useEffect,useMemo}from'react';import moment from'moment';import {Spin,Row,Col,Button,Card,message}from'antd';import {useGetDurationTimeRecordsQuery,useGetDurationTimesheetsQuery,useCreateTimeRecordMutation,useUpdateTimeRecordMutation,useRemoveDurationTimeRecordsMutation,useCreateTimesheetMutation}from'@admin-layout/timetracker-core/lib/generated/react-hooks';import {useSplitTimeRecordsPerDay}from'../../../hooks/index.js';import {TabularCalendar}from'./TabularCalendar.js';import {TabularDayView}from'./TabularDayView.js';import {view}from'../index.js';import*as _ from'lodash';const filterTimeRecords = (records, filterOptions) => {
    const { selectedUser, selectedProject, approval, members } = filterOptions;
    if (!records)
        return [];
    const memStrArr = members.map((mem) => mem.userId);
    return records
        .filter((ev) => (ev.userId === selectedUser || selectedUser === '__all') &&
        (ev.projectId === selectedProject || selectedProject === '__all') &&
        (approval === undefined ? true : approval ? !!ev.timesheetId : !ev.timesheetId))
        .filter((record) => memStrArr.includes(record.userId));
};
const TabularCalendarWrapper = ({ projects, weekStart, selectedUser, members, selectedProject, setPathWeekStart, }) => {
    const [timeRecords, setTimeRecords] = useState([]);
    const [projectsApproval, setProjectsApproval] = useState({ approvals: [], unApprovals: [] });
    const [projectsMap, setProjectsMap] = useState(null);
    const [memTimesheet, setMemTimesheet] = useState([]);
    const [filteredRecords, setFilteredRecords] = useState([]);
    const [tabularView, setTabularView] = useState(view.TABULAR_WEEK);
    const [selectedDay, setSelectedDay] = useState(moment(weekStart));
    const { data: { getDurationTimeRecords } = {}, loading: loadingRecords, refetch, error, } = useGetDurationTimeRecordsQuery({
        variables: {
            startTime: weekStart,
            endTime: moment(weekStart).add(1, 'week'),
        },
    });
    const records = useSplitTimeRecordsPerDay(getDurationTimeRecords);
    const { data: approvalData, loading: loadingApproval, refetch: refetchApproval, } = useGetDurationTimesheetsQuery({
        variables: {
            start: moment(weekStart),
            end: moment(weekStart).add(1, 'week'),
        },
    });
    const reloadData = () => {
        refetch();
        refetchApproval();
    };
    useEffect(() => {
        reloadData();
    }, [weekStart]);
    useEffect(() => {
        setTimeRecords(records.map((record) => (Object.assign(Object.assign({}, record), { isSplit: records.filter((data) => data.id === record.id).length > 1 }))));
    }, [records]);
    useEffect(() => {
        const projectsMap = new Map();
        projects.forEach((project) => {
            projectsMap.set(project.id, project);
        });
        setProjectsMap(projectsMap);
    }, [projects]);
    useEffect(() => {
        setMemTimesheet(memberTimesheets(selectedUser));
    }, [selectedUser, approvalData]);
    useEffect(() => {
        const records = filterTimeRecords(timeRecords, {
            selectedUser,
            selectedProject,
            members,
        });
        setFilteredRecords(records.map((record) => (Object.assign(Object.assign({}, record), { projectId: record.projectId || '' }))));
        let approvedSet = new Set();
        let unApprovedSet = new Set();
        records.forEach((record) => {
            if (!!record.timesheetId)
                approvedSet.add(record.projectId || '');
            else
                unApprovedSet.add(record.projectId || '');
        });
        setProjectsApproval({
            approvals: Array.from(approvedSet.values()),
            unApprovals: Array.from(unApprovedSet.values()),
        });
    }, [timeRecords, selectedUser, selectedProject, members]);
    const [createMutation, { loading: loadingCreate }] = useCreateTimeRecordMutation();
    const [updateMutation, { loading: loadingUpdate }] = useUpdateTimeRecordMutation();
    const [removeMutation, { loading: loadingRemove }] = useRemoveDurationTimeRecordsMutation();
    const [createTimesheetMutation, { loading: loadingCreateTimesheet }] = useCreateTimesheetMutation();
    const handleRemoveDuration = (pId) => {
        removeMutation({
            variables: {
                startTime: moment(weekStart),
                endTime: moment(weekStart).add(1, 'week'),
                projectId: pId,
            },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('Removed');
                }
                else {
                    console.log(errors[0].message);
                }
            }
        });
    };
    // create time record
    const createTimeRecord = (request) => {
        createMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('TimeRecord created');
                }
                else {
                    message.error(error.message);
                }
            },
        });
    };
    // update time record
    const updateTimeRecord = (recordId, request) => {
        updateMutation({
            variables: { recordId, request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('TimeRecord Updated');
                }
                else {
                    message.error(error.message);
                }
            }
        });
    };
    // create timeSheet
    const createTimesheet = (request) => {
        createTimesheetMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimesheets() { }
                        }
                    });
                    message.success('Timesheet Created');
                }
                else {
                    message.error(error.message);
                }
            },
        });
    };
    const memberTimesheets = (userId) => {
        return _.get(approvalData, 'getDurationTimesheets', []).filter((sheet) => sheet.userId === userId);
    };
    const onClickBack = (event) => {
        event.preventDefault();
        if (tabularView === view.TABULAR_DAY) {
            setSelectedDay(moment(selectedDay).subtract(1, 'd'));
        }
        else {
            const newWeekStart = moment(weekStart).add('-1', 'week');
            setPathWeekStart(newWeekStart);
        }
    };
    const onClickNext = (event) => {
        event.preventDefault();
        if (tabularView === view.TABULAR_DAY) {
            setSelectedDay(moment(selectedDay).add(1, 'd'));
        }
        else {
            const newWeekStart = moment(weekStart).add('1', 'week');
            setPathWeekStart(newWeekStart);
        }
    };
    const onClickToday = (event) => {
        event.preventDefault();
        if (tabularView === view.TABULAR_DAY) {
            setSelectedDay(moment());
        }
        else {
            const newWeekStart = moment().startOf('week');
            setPathWeekStart(newWeekStart);
        }
    };
    const onClickWeek = () => {
        setTabularView(view.TABULAR_WEEK);
        setPathWeekStart(moment(selectedDay).startOf('week'));
    };
    const onClickDay = () => {
        setTabularView(view.TABULAR_DAY);
        setSelectedDay(moment(weekStart));
    };
    return useMemo(() => {
        return (React__default.createElement(Spin, { spinning: !getDurationTimeRecords ||
                loadingRecords ||
                loadingCreate ||
                loadingUpdate ||
                loadingRemove ||
                loadingCreateTimesheet },
            React__default.createElement(Row, { className: "toolBar" },
                React__default.createElement(Col, { xs: 24, md: 6, className: "control" },
                    React__default.createElement(Button, { onClick: onClickToday }, " Today "),
                    React__default.createElement(Button, { onClick: onClickBack }, " Back "),
                    React__default.createElement(Button, { onClick: onClickNext }, " Next ")),
                React__default.createElement(Col, { xs: 24, md: 12, style: { textAlign: 'center' } }, tabularView === view.TABULAR_DAY ? (React__default.createElement("span", { style: { fontWeight: 500 } }, selectedDay.format('Do MMM, YYYY'))) : (React__default.createElement(React__default.Fragment, null,
                    React__default.createElement("span", { className: "duration-start", style: { fontWeight: 500 } }, moment(weekStart).format('MMMM DD')),
                    " -",
                    React__default.createElement("span", { className: "duration-end", style: { fontWeight: 500 } }, moment(weekStart).format('MM') === moment(weekStart).add(1, 'week').format('MM')
                        ? moment(weekStart).add(1, 'week').format('DD')
                        : moment(weekStart).add(1, 'week').format('MMMM DD'))))),
                React__default.createElement(Col, { xs: 24, md: 6, className: "control", style: { textAlign: 'right' } },
                    React__default.createElement(Button, { onClick: onClickDay }, " Day "),
                    React__default.createElement(Button, { onClick: onClickWeek }, " Week "),
                    React__default.createElement(Button, { onClick: () => setTabularView(view.TABULAR_MONTH) }, " Month "))),
            React__default.createElement(Row, null,
                React__default.createElement(Col, { xs: 24 },
                    React__default.createElement(Card, null, (selectedUser !== '__all' ? (tabularView === view.TABULAR_DAY ? (React__default.createElement(TabularDayView, { members: members, projects: projects, selectedDay: selectedDay, userId: selectedUser })) : (React__default.createElement(TabularCalendar, { weekStart: weekStart, records: filteredRecords, projects: projects, projectsMap: projectsMap, projectsApproval: projectsApproval, timesheets: memTimesheet, selectedUser: selectedUser, members: members, loading: loadingRecords ||
                            loadingCreate ||
                            loadingUpdate ||
                            loadingRemove ||
                            loadingCreateTimesheet, handleRemoveDuration: handleRemoveDuration, createTimeRecord: createTimeRecord, updateTimeRecord: updateTimeRecord, createTimesheet: createTimesheet }))) : (members.map((mem) => {
                        return (tabularView === view.TABULAR_DAY ? (React__default.createElement(TabularDayView, { members: members, projects: projects, selectedDay: selectedDay, userId: mem.userId })) : (React__default.createElement(TabularCalendar, { key: mem.userId, weekStart: weekStart, records: filteredRecords.filter((record) => (record.userId === mem.userId)), projects: projects, projectsMap: projectsMap, projectsApproval: projectsApproval, timesheets: memberTimesheets(mem.userId), selectedUser: mem.userId, members: members, loading: loadingRecords ||
                                loadingCreate ||
                                loadingUpdate ||
                                loadingRemove ||
                                loadingCreateTimesheet, handleRemoveDuration: handleRemoveDuration, createTimeRecord: createTimeRecord, updateTimeRecord: updateTimeRecord, createTimesheet: createTimesheet })));
                    }))))))));
    }, [
        getDurationTimeRecords,
        loadingRecords,
        loadingCreate,
        loadingUpdate,
        loadingRemove,
        loadingCreateTimesheet,
        weekStart,
        filteredRecords,
        projects,
        projectsMap,
        projectsApproval,
        members,
        memTimesheet,
        selectedUser,
        approvalData,
        tabularView,
        selectedDay
    ]);
};export default TabularCalendarWrapper;//# sourceMappingURL=index.js.map
